@@ -22,6 +22,7 @@
 #include <botan/internal/cpuid.h>
 #include <botan/internal/fmt.h>
 #include <botan/internal/os_utils.h>
+#include <botan/internal/stl_util.h>
 #include <botan/internal/timer.h>
 
 #if defined(BOTAN_HAS_BIGINT)
@@ -108,6 +109,10 @@
    #include <botan/ec_group.h>
 #endif
 
+#if defined(BOTAN_HAS_PCURVES)
+   #include <botan/internal/pcurves.h>
+#endif
+
 #if defined(BOTAN_HAS_DL_GROUP)
    #include <botan/dl_group.h>
 #endif
@@ -124,6 +129,10 @@
    #include <botan/dilithium.h>
 #endif
 
+#if defined(BOTAN_HAS_HSS_LMS)
+   #include <botan/hss_lms.h>
+#endif
+
 #if defined(BOTAN_HAS_SPHINCS_PLUS_WITH_SHA2) || defined(BOTAN_HAS_SPHINCS_PLUS_WITH_SHAKE)
    #include <botan/sphincsplus.h>
 #endif
@@ -406,12 +415,13 @@ class Speed final : public Command {
             "ECDSA",
             "Ed25519",
             "Ed448",
-            "Curve25519",
+            "X25519",
             "X448",
             "McEliece",
             "Kyber",
             "SPHINCS+",
-            "FrodoKEM"
+            "FrodoKEM",
+            "HSS-LMS",
          };
          // clang-format on
       }
@@ -458,7 +468,7 @@ class Speed final : public Command {
 
 #if defined(BOTAN_HAS_ECC_GROUP)
          if(ecc_groups.empty()) {
-            ecc_groups = {"secp256r1", "brainpool256r1", "secp384r1", "brainpool384r1", "secp521r1", "brainpool512r1"};
+            ecc_groups = {"secp256r1", "secp384r1", "secp521r1", "brainpool256r1", "brainpool384r1", "brainpool512r1"};
          } else if(ecc_groups.size() == 1 && ecc_groups[0] == "all") {
             auto all = Botan::EC_Group::known_named_groups();
             ecc_groups.assign(all.begin(), all.end());
@@ -539,6 +549,17 @@ class Speed final : public Command {
                bench_rsa_keygen(provider, msec);
             }
 #endif
+
+#if defined(BOTAN_HAS_PCURVES)
+            else if(algo == "ECDSA-pcurves") {
+               bench_pcurve_ecdsa(ecc_groups, msec);
+            } else if(algo == "ECDH-pcurves") {
+               bench_pcurve_ecdh(ecc_groups, msec);
+            } else if(algo == "pcurves") {
+               bench_pcurves(ecc_groups, msec);
+            }
+#endif
+
 #if defined(BOTAN_HAS_ECDSA)
             else if(algo == "ECDSA") {
                bench_ecdsa(ecc_groups, provider, msec);
@@ -596,9 +617,9 @@ class Speed final : public Command {
                bench_ecdh(ecc_groups, provider, msec);
             }
 #endif
-#if defined(BOTAN_HAS_CURVE_25519)
-            else if(algo == "Curve25519") {
-               bench_curve25519(provider, msec);
+#if defined(BOTAN_HAS_X25519)
+            else if(algo == "X25519") {
+               bench_x25519(provider, msec);
             }
 #endif
 #if defined(BOTAN_HAS_X448)
@@ -626,6 +647,11 @@ class Speed final : public Command {
                bench_xmss(provider, msec);
             }
 #endif
+#if defined(BOTAN_HAS_HSS_LMS)
+            else if(algo == "HSS-LMS") {
+               bench_hss_lms(provider, msec);
+            }
+#endif
 #if defined(BOTAN_HAS_SPHINCS_PLUS_WITH_SHA2) || defined(BOTAN_HAS_SPHINCS_PLUS_WITH_SHAKE)
             else if(algo == "SPHINCS+") {
                bench_sphincs_plus(provider, msec);
@@ -712,8 +738,6 @@ class Speed final : public Command {
 #if defined(BOTAN_HAS_ECC_GROUP)
             else if(algo == "ecc_mult") {
                bench_ecc_mult(ecc_groups, msec);
-            } else if(algo == "ecc_ops") {
-               bench_ecc_ops(ecc_groups, msec);
             } else if(algo == "ecc_init") {
                bench_ecc_init(ecc_groups, msec);
             } else if(algo == "os2ecp") {
@@ -1074,42 +1098,13 @@ class Speed final : public Command {
       }
 
 #if defined(BOTAN_HAS_ECC_GROUP)
-      void bench_ecc_ops(const std::vector<std::string>& groups, const std::chrono::milliseconds runtime) {
-         for(const std::string& group_name : groups) {
-            const Botan::EC_Group ec_group(group_name);
-
-            auto add_timer = make_timer(group_name + " add");
-            auto addf_timer = make_timer(group_name + " addf");
-            auto dbl_timer = make_timer(group_name + " dbl");
-
-            const Botan::EC_Point& base_point = ec_group.get_base_point();
-
-            // create a non-affine point
-            const auto random_k = Botan::BigInt::from_u64(0x4E6F537465707E);
-            Botan::EC_Point non_affine_pt = ec_group.get_base_point() * random_k;
-            Botan::EC_Point pt = ec_group.get_base_point();
-
-            std::vector<Botan::BigInt> ws(Botan::EC_Point::WORKSPACE_SIZE);
-
-            while(add_timer->under(runtime) && addf_timer->under(runtime) && dbl_timer->under(runtime)) {
-               dbl_timer->run([&]() { pt.mult2(ws); });
-               add_timer->run([&]() { pt.add(non_affine_pt, ws); });
-               addf_timer->run([&]() { pt.add_affine(base_point, ws); });
-            }
-
-            record_result(dbl_timer);
-            record_result(add_timer);
-            record_result(addf_timer);
-         }
-      }
-
       void bench_ecc_init(const std::vector<std::string>& groups, const std::chrono::milliseconds runtime) {
          for(std::string group_name : groups) {
             auto timer = make_timer(group_name + " initialization");
 
             while(timer->under(runtime)) {
                Botan::EC_Group::clear_registered_curve_data();
-               timer->run([&]() { Botan::EC_Group group(group_name); });
+               timer->run([&]() { Botan::EC_Group::from_name(group_name); });
             }
 
             record_result(timer);
@@ -1118,7 +1113,7 @@ class Speed final : public Command {
 
       void bench_ecc_mult(const std::vector<std::string>& groups, const std::chrono::milliseconds runtime) {
          for(const std::string& group_name : groups) {
-            const Botan::EC_Group ec_group(group_name);
+            const auto ec_group = Botan::EC_Group::from_name(group_name);
 
             auto mult_timer = make_timer(group_name + " Montgomery ladder");
             auto blinded_mult_timer = make_timer(group_name + " blinded comb");
@@ -1130,15 +1125,21 @@ class Speed final : public Command {
 
             while(mult_timer->under(runtime) && blinded_mult_timer->under(runtime) &&
                   blinded_var_mult_timer->under(runtime)) {
-               const Botan::BigInt scalar(rng(), ec_group.get_p_bits());
+               const Botan::BigInt scalar = ec_group.random_scalar(rng());
 
                const Botan::EC_Point r1 = mult_timer->run([&]() { return base_point * scalar; });
 
-               const Botan::EC_Point r2 =
-                  blinded_mult_timer->run([&]() { return ec_group.blinded_base_point_multiply(scalar, rng(), ws); });
+               const Botan::EC_Point r2 = blinded_mult_timer->run([&]() {
+                  auto pt = ec_group.blinded_base_point_multiply(scalar, rng(), ws);
+                  pt.force_affine();
+                  return pt;
+               });
 
-               const Botan::EC_Point r3 = blinded_var_mult_timer->run(
-                  [&]() { return ec_group.blinded_var_point_multiply(base_point, scalar, rng(), ws); });
+               const Botan::EC_Point r3 = blinded_var_mult_timer->run([&]() {
+                  auto pt = ec_group.blinded_var_point_multiply(base_point, scalar, rng(), ws);
+                  pt.force_affine();
+                  return pt;
+               });
 
                BOTAN_ASSERT_EQUAL(r1, r2, "Same point computed by Montgomery and comb");
                BOTAN_ASSERT_EQUAL(r1, r3, "Same point computed by Montgomery and window");
@@ -1155,7 +1156,7 @@ class Speed final : public Command {
             auto uncmp_timer = make_timer("OS2ECP uncompressed " + group_name);
             auto cmp_timer = make_timer("OS2ECP compressed " + group_name);
 
-            const Botan::EC_Group ec_group(group_name);
+            const auto ec_group = Botan::EC_Group::from_name(group_name);
 
             while(uncmp_timer->under(runtime) && cmp_timer->under(runtime)) {
                const Botan::BigInt k(rng(), 256);
@@ -1180,7 +1181,7 @@ class Speed final : public Command {
             auto h2c_ro_timer = make_timer(group_name + "-RO", "", "hash to curve");
             auto h2c_nu_timer = make_timer(group_name + "-NU", "", "hash to curve");
 
-            const Botan::EC_Group group(group_name);
+            const auto group = Botan::EC_Group::from_name(group_name);
 
             while(h2c_ro_timer->under(runtime)) {
                std::vector<uint8_t> input(32);
@@ -1204,6 +1205,189 @@ class Speed final : public Command {
       }
 #endif
 
+#if defined(BOTAN_HAS_PCURVES)
+
+      void bench_pcurves(const std::vector<std::string>& groups, const std::chrono::milliseconds runtime) {
+         for(const auto& group_name : groups) {
+            if(auto curve = Botan::PCurve::PrimeOrderCurve::from_name(group_name)) {
+               auto base_timer = make_timer(group_name + " pcurve base mul");
+               auto var_timer = make_timer(group_name + " pcurve var mul");
+               auto mul2_setup_timer = make_timer(group_name + " pcurve mul2 setup");
+               auto mul2_timer = make_timer(group_name + " pcurve mul2");
+
+               auto scalar_invert = make_timer(group_name + " pcurve scalar invert");
+               auto to_affine = make_timer(group_name + " pcurve proj->affine");
+
+               auto g = curve->generator();
+               auto h = curve->mul(g, curve->random_scalar(rng()), rng()).to_affine();
+               auto gh_tab = curve->mul2_setup(g, h);
+
+               while(base_timer->under(runtime)) {
+                  const auto scalar = curve->random_scalar(rng());
+                  base_timer->run([&]() { return curve->mul_by_g(scalar, rng()).to_affine(); });
+               }
+
+               while(var_timer->under(runtime)) {
+                  const auto scalar = curve->random_scalar(rng());
+                  var_timer->run([&]() { return curve->mul(h, scalar, rng()).to_affine(); });
+               }
+
+               while(mul2_setup_timer->under(runtime)) {
+                  mul2_setup_timer->run([&]() { return curve->mul2_setup(g, h); });
+               }
+
+               while(mul2_timer->under(runtime)) {
+                  const auto scalar = curve->random_scalar(rng());
+                  const auto scalar2 = curve->random_scalar(rng());
+                  mul2_timer->run([&]() -> std::optional<Botan::PCurve::PrimeOrderCurve::AffinePoint> {
+                     if(auto pt = curve->mul2_vartime(*gh_tab, scalar, scalar2)) {
+                        return pt->to_affine();
+                     } else {
+                        return {};
+                     }
+                  });
+               }
+
+               auto pt = curve->mul(g, curve->random_scalar(rng()), rng());
+               to_affine->run_until_elapsed(runtime, [&]() { pt.to_affine(); });
+
+               while(scalar_invert->under(runtime)) {
+                  const auto scalar = curve->random_scalar(rng());
+                  scalar_invert->run([&]() { scalar.invert(); });
+               }
+
+               record_result(base_timer);
+               record_result(var_timer);
+               record_result(mul2_setup_timer);
+               record_result(mul2_timer);
+               record_result(to_affine);
+               record_result(scalar_invert);
+            }
+         }
+      }
+
+      void bench_pcurve_ecdsa(const std::vector<std::string>& groups, const std::chrono::milliseconds runtime) {
+         for(const auto& group_name : groups) {
+            auto curve = Botan::PCurve::PrimeOrderCurve::from_name(group_name);
+            if(!curve) {
+               continue;
+            }
+
+            // Setup (not timed)
+            const auto g = curve->generator();
+            const auto x = curve->random_scalar(rng());
+            const auto y = curve->mul_by_g(x, rng()).to_affine();
+            const auto e = curve->random_scalar(rng());
+
+            const auto gy_tab = curve->mul2_setup(g, y);
+
+            auto b = curve->random_scalar(rng());
+            auto b_inv = b.invert();
+
+            auto sign_timer = make_timer("ECDSA sign pcurves " + group_name);
+            auto verify_timer = make_timer("ECDSA verify pcurves " + group_name);
+
+            while(sign_timer->under(runtime)) {
+               sign_timer->start();
+
+               const auto signature = [&]() {
+                  const auto k = curve->random_scalar(rng());
+                  const auto r = curve->base_point_mul_x_mod_order(k, rng());
+                  const auto k_inv = k.invert();
+                  b = b.square();
+                  b_inv = b_inv.square();
+                  const auto be = b * e;
+                  const auto bx = b * x;
+                  const auto bxr_e = (bx * r) + be;
+                  const auto s = (k_inv * bxr_e) * b_inv;
+
+                  return Botan::concat(r.serialize(), s.serialize());
+               }();
+
+               sign_timer->stop();
+
+               verify_timer->start();
+
+               auto result = [&](std::span<const uint8_t> sig) {
+                  const size_t scalar_bytes = curve->scalar_bytes();
+                  if(sig.size() != 2 * scalar_bytes) {
+                     return false;
+                  }
+
+                  const auto r = curve->deserialize_scalar(sig.first(scalar_bytes));
+                  const auto s = curve->deserialize_scalar(sig.last(scalar_bytes));
+
+                  if(r && s) {
+                     if(r->is_zero() || s->is_zero()) {
+                        return false;
+                     }
+
+                     auto w = s->invert();
+
+                     auto u1 = e * w;
+                     auto u2 = *r * w;
+
+                     auto v = curve->mul2_vartime_x_mod_order(*gy_tab, u1, u2);
+
+                     return (r == v);
+                  }
+
+                  return false;
+               }(signature);
+
+               BOTAN_ASSERT(result, "ECDSA-pcurves signature ok");
+
+               verify_timer->stop();
+            }
+
+            record_result(sign_timer);
+            record_result(verify_timer);
+         }
+      }
+
+      void bench_pcurve_ecdh(const std::vector<std::string>& groups, const std::chrono::milliseconds runtime) {
+         for(const auto& group_name : groups) {
+            auto curve = Botan::PCurve::PrimeOrderCurve::from_name(group_name);
+            if(!curve) {
+               continue;
+            }
+
+            auto ka_timer = make_timer("ECDH agree pcurves " + group_name);
+
+            auto agree = [&](const Botan::PCurve::PrimeOrderCurve::Scalar& sk, std::span<const uint8_t> pt_bytes) {
+               const auto pt = curve->deserialize_point(pt_bytes);
+               if(pt) {
+                  return curve->mul(*pt, sk, rng()).to_affine().serialize();
+               } else {
+                  return std::vector<uint8_t>();
+               }
+            };
+
+            while(ka_timer->under(runtime)) {
+               const auto g = curve->generator();
+               const auto x1 = curve->random_scalar(rng());
+               const auto x2 = curve->random_scalar(rng());
+
+               const auto y1 = curve->mul_by_g(x1, rng()).to_affine().serialize();
+               const auto y2 = curve->mul_by_g(x2, rng()).to_affine().serialize();
+
+               ka_timer->start();
+               const auto ss1 = agree(x1, y2);
+               ka_timer->stop();
+
+               ka_timer->start();
+               const auto ss2 = agree(x1, y2);
+               ka_timer->stop();
+
+               BOTAN_ASSERT(ss1 == ss2, "Key agreement worked");
+            }
+
+            record_result(ka_timer);
+         }
+      }
+
+#endif
+
 #if defined(BOTAN_HAS_FPE_FE1)
 
       void bench_fpe_fe1(const std::chrono::milliseconds runtime) {
@@ -1793,7 +1977,7 @@ class Speed final : public Command {
                                 const std::string& /*unused*/,
                                 std::chrono::milliseconds msec) {
          for(const std::string& group_name : groups) {
-            Botan::EC_Group group(group_name);
+            const auto group = Botan::EC_Group::from_name(group_name);
             auto recovery_timer = make_timer("ECDSA recovery " + group_name);
 
             while(recovery_timer->under(msec)) {
@@ -1925,9 +2109,9 @@ class Speed final : public Command {
       }
 #endif
 
-#if defined(BOTAN_HAS_CURVE_25519)
-      void bench_curve25519(const std::string& provider, std::chrono::milliseconds msec) {
-         bench_pk_ka("Curve25519", "Curve25519", "", provider, msec);
+#if defined(BOTAN_HAS_X25519)
+      void bench_x25519(const std::string& provider, std::chrono::milliseconds msec) {
+         bench_pk_ka("X25519", "X25519", "", provider, msec);
       }
 #endif
 
@@ -2134,6 +2318,33 @@ class Speed final : public Command {
       }
 #endif
 
+#if defined(BOTAN_HAS_HSS_LMS)
+      void bench_hss_lms(const std::string& provider, std::chrono::milliseconds msec) {
+         // At first we compare instances with multiple hash functions. LMS trees with
+         // height 10 are suitable, since they can be used for enough signatures and are
+         // fast enough for speed testing.
+         // Afterward, setups with multiple HSS layers are tested
+         std::vector<std::string> hss_lms_instances{"SHA-256,HW(10,1)",
+                                                    "SHAKE-256(256),HW(10,1)",
+                                                    "SHAKE-256(192),HW(10,1)",
+                                                    "Truncated(SHA-256,192),HW(10,1)",
+                                                    "SHA-256,HW(10,1),HW(10,1)",
+                                                    "SHA-256,HW(10,1),HW(10,1),HW(10,1)"};
+
+         for(const auto& params : hss_lms_instances) {
+            auto keygen_timer = make_timer(params, provider, "keygen");
+
+            std::unique_ptr<Botan::Private_Key> key(
+               keygen_timer->run([&] { return Botan::create_private_key("HSS-LMS", rng(), params); }));
+
+            record_result(keygen_timer);
+            if(bench_pk_sig(*key, params, provider, "", msec) == 1) {
+               break;
+            }
+         }
+      }
+#endif
+
 #if defined(BOTAN_HAS_ZFEC)
       void bench_zfec(std::chrono::milliseconds msec) {
          const size_t k = 4;
