@@ -260,7 +260,9 @@ class ECDSA_Timing_Test final : public Timing_Test {
 };
 
 ECDSA_Timing_Test::ECDSA_Timing_Test(const std::string& ecgroup) :
-      m_group(ecgroup), m_privkey(timing_test_rng(), m_group), m_x(m_privkey.private_value()) {
+      m_group(Botan::EC_Group::from_name(ecgroup)),
+      m_privkey(timing_test_rng(), m_group),
+      m_x(m_privkey.private_value()) {
    m_b = m_group.random_scalar(timing_test_rng());
    m_b_inv = m_group.inverse_mod_order(m_b);
 }
@@ -296,7 +298,7 @@ ticks ECDSA_Timing_Test::measure_critical_function(const std::vector<uint8_t>& i
 
 class ECC_Mul_Timing_Test final : public Timing_Test {
    public:
-      explicit ECC_Mul_Timing_Test(const std::string& ecgroup) : m_group(ecgroup) {}
+      explicit ECC_Mul_Timing_Test(const std::string& ecgroup) : m_group(Botan::EC_Group::from_name(ecgroup)) {}
 
       ticks measure_critical_function(const std::vector<uint8_t>& input) override;
 
@@ -620,13 +622,7 @@ class MARVIN_Test_Command final : public Command {
 
                const uint64_t start = Botan::OS::get_system_timestamp_ns();
 
-   #if 0
-               try {
-                  op.decrypt(ciphertext.data(), modulus_bytes);
-               } catch(...) {}
-   #else
                op.decrypt_or_random(ciphertext.data(), modulus_bytes, expect_pt_len, rng());
-   #endif
 
                const uint64_t duration = Botan::OS::get_system_timestamp_ns() - start;
                BOTAN_ASSERT_NOMSG(measurements[testcase].size() == r);
