@@ -58,6 +58,15 @@ class Hybrid_PublicKey : public virtual Botan::Public_Key {
          throw Botan::Not_Implemented("Hybrid-KEM does not have an algorithm identifier");
       }
 
+      /**
+       * In an actual implementation, this may return a serialized
+       * representation of the public keys. For instance, a mere concatenation
+       * of the two public keys.
+       */
+      std::vector<uint8_t> raw_public_key_bits() const override {
+         throw Botan::Not_Implemented("Raw key serialization is not supported");
+      }
+
       /**
        * In an actual implementation, this may return a serialized
        * representation of the public keys. For instance, using some ASN.1
@@ -161,7 +170,7 @@ class Hybrid_Encryption_Operation : public Botan::PK_Ops::KEM_Encryption {
        * key (ephemeral key pair) and the length of the KEM's encapsulated key.
        */
       size_t encapsulated_key_length() const override {
-         return m_hybrid_pk.kex_public_key().public_key_bits().size() + m_kem_encryptor.encapsulated_key_length();
+         return m_hybrid_pk.kex_public_key().raw_public_key_bits().size() + m_kem_encryptor.encapsulated_key_length();
       }
 
       /**
@@ -212,8 +221,8 @@ class Hybrid_Encryption_Operation : public Botan::PK_Ops::KEM_Encryption {
          // TODO: kex.derive_key() should allow a std::span<>-based out param,
          //       which would save a copy in this case. (See GH #3318)
          const auto kex_shared_key =
-            kex.derive_key(0 /* no KDF */, m_hybrid_pk.kex_public_key().public_key_bits()).bits_of();
-         const auto kex_encapsed_key = ephemeral_keypair->public_key_bits();
+            kex.derive_key(0 /* no KDF */, m_hybrid_pk.kex_public_key().raw_public_key_bits()).bits_of();
+         const auto kex_encapsed_key = ephemeral_keypair->raw_public_key_bits();
 
          // 3. KEX: Encapsulate a shared secret using the KEM's public key,
          //         yielding a shared secret and its encapsulation.
@@ -266,7 +275,7 @@ class Hybrid_Decryption_Operation : public Botan::PK_Ops::KEM_Decryption {
        * key (ephemeral key pair) and the length of the KEM's encapsulated key.
        */
       size_t encapsulated_key_length() const override {
-         return m_hybrid_sk.kex_public_key().public_key_bits().size() + m_kem_decryptor.encapsulated_key_length();
+         return m_hybrid_sk.kex_public_key().raw_public_key_bits().size() + m_kem_decryptor.encapsulated_key_length();
       }
 
       /**
@@ -298,7 +307,7 @@ class Hybrid_Decryption_Operation : public Botan::PK_Ops::KEM_Decryption {
 
          // 1. Hybrid: Extract the ephemeral public key and the encapsulation.
          const auto kex_encapsed_key =
-            encapsulated_key.subspan(0, m_hybrid_sk.kex_public_key().public_key_bits().size());
+            encapsulated_key.subspan(0, m_hybrid_sk.kex_public_key().raw_public_key_bits().size());
          const auto kem_encapsed_key = encapsulated_key.subspan(kex_encapsed_key.size());
 
          // 2. KEX: Agree on a shared secret using the KEX's private key and the
@@ -351,7 +360,7 @@ int main() {
    // She then combines them into a custom "hybrid" key pair that acts
    // like a key encapsulation mechanism (KEM).
    const auto private_key_of_alice = std::make_unique<Hybrid_PrivateKey>(
-      Botan::create_private_key("Curve25519", rng), Botan::create_private_key("Kyber", rng, "Kyber-768-r3"));
+      Botan::create_private_key("X25519", rng), Botan::create_private_key("Kyber", rng, "Kyber-768-r3"));
    const auto public_key_of_alice = private_key_of_alice->public_key();
 
    // Bob uses Alice's public key to encapsulate a shared secret, and
