@@ -69,7 +69,7 @@ BigInt::BigInt(std::string_view str) {
    size_t markers = 0;
    bool negative = false;
 
-   if(str.length() > 0 && str[0] == '-') {
+   if(!str.empty() && str[0] == '-') {
       markers += 1;
       negative = true;
    }
@@ -88,8 +88,14 @@ BigInt::BigInt(std::string_view str) {
    }
 }
 
-BigInt::BigInt(const uint8_t input[], size_t length) {
-   binary_decode(input, length);
+BigInt BigInt::from_string(std::string_view str) {
+   return BigInt(str);
+}
+
+BigInt BigInt::from_bytes(std::span<const uint8_t> input) {
+   BigInt r;
+   r.assign_from_bytes(input);
+   return r;
 }
 
 /*
@@ -103,8 +109,7 @@ BigInt::BigInt(const uint8_t input[], size_t length, Base base) {
 BigInt BigInt::from_bytes_with_max_bits(const uint8_t input[], size_t length, size_t max_bits) {
    const size_t input_bits = 8 * length;
 
-   BigInt bn;
-   bn.binary_decode(input, length);
+   auto bn = BigInt::from_bytes(std::span{input, length});
 
    if(input_bits > max_bits) {
       const size_t bits_to_shift = input_bits - max_bits;
@@ -136,7 +141,7 @@ int32_t BigInt::cmp_word(word other) const {
       return 1;  // must be larger since other is just one word ...
    }
 
-   return bigint_cmp(this->data(), sw, &other, 1);
+   return bigint_cmp(this->_data(), sw, &other, 1);
 }
 
 /*
@@ -153,11 +158,11 @@ int32_t BigInt::cmp(const BigInt& other, bool check_signs) const {
       }
 
       if(other.is_negative() && this->is_negative()) {
-         return (-bigint_cmp(this->data(), this->size(), other.data(), other.size()));
+         return (-bigint_cmp(this->_data(), this->size(), other._data(), other.size()));
       }
    }
 
-   return bigint_cmp(this->data(), this->size(), other.data(), other.size());
+   return bigint_cmp(this->_data(), this->size(), other._data(), other.size());
 }
 
 bool BigInt::is_equal(const BigInt& other) const {
@@ -165,7 +170,7 @@ bool BigInt::is_equal(const BigInt& other) const {
       return false;
    }
 
-   return bigint_ct_is_eq(this->data(), this->sig_words(), other.data(), other.sig_words()).as_bool();
+   return bigint_ct_is_eq(this->_data(), this->sig_words(), other._data(), other.sig_words()).as_bool();
 }
 
 bool BigInt::is_less_than(const BigInt& other) const {
@@ -178,10 +183,10 @@ bool BigInt::is_less_than(const BigInt& other) const {
    }
 
    if(other.is_negative() && this->is_negative()) {
-      return bigint_ct_is_lt(other.data(), other.sig_words(), this->data(), this->sig_words()).as_bool();
+      return bigint_ct_is_lt(other._data(), other.sig_words(), this->_data(), this->sig_words()).as_bool();
    }
 
-   return bigint_ct_is_lt(this->data(), this->sig_words(), other.data(), other.sig_words()).as_bool();
+   return bigint_ct_is_lt(this->_data(), this->sig_words(), other._data(), other.sig_words()).as_bool();
 }
 
 void BigInt::encode_words(word out[], size_t size) const {
@@ -192,7 +197,7 @@ void BigInt::encode_words(word out[], size_t size) const {
    }
 
    clear_mem(out, size);
-   copy_mem(out, data(), words);
+   copy_mem(out, _data(), words);
 }
 
 size_t BigInt::Data::calc_sig_words() const {
@@ -282,7 +287,7 @@ size_t BigInt::top_bits_free() const {
    const size_t words = sig_words();
 
    const word top_word = word_at(words - 1);
-   const size_t bits_used = high_bit(top_word);
+   const size_t bits_used = high_bit(CT::value_barrier(top_word));
    CT::unpoison(bits_used);
    return BOTAN_MP_WORD_BITS - bits_used;
 }
@@ -329,7 +334,7 @@ size_t BigInt::reduce_below(const BigInt& p, secure_vector<word>& ws) {
    size_t reductions = 0;
 
    for(;;) {
-      word borrow = bigint_sub3(ws.data(), data(), p_words + 1, p.data(), p_words);
+      word borrow = bigint_sub3(ws.data(), _data(), p_words + 1, p._data(), p_words);
       if(borrow) {
          break;
       }
@@ -357,9 +362,9 @@ void BigInt::ct_reduce_below(const BigInt& mod, secure_vector<word>& ws, size_t
    clear_mem(ws.data(), sz);
 
    for(size_t i = 0; i != bound; ++i) {
-      word borrow = bigint_sub3(ws.data(), data(), sz, mod.data(), mod_words);
+      word borrow = bigint_sub3(ws.data(), _data(), sz, mod._data(), mod_words);
 
-      CT::Mask<word>::is_zero(borrow).select_n(mutable_data(), ws.data(), data(), sz);
+      CT::Mask<word>::is_zero(borrow).select_n(mutable_data(), ws.data(), _data(), sz);
    }
 }
 
@@ -372,8 +377,13 @@ BigInt BigInt::abs() const {
    return x;
 }
 
-void BigInt::binary_encode(uint8_t buf[]) const {
-   this->binary_encode(buf, bytes());
+/*
+* Encode this number into bytes
+*/
+void BigInt::serialize_to(std::span<uint8_t> output) const {
+   BOTAN_ARG_CHECK(this->bytes() <= output.size(), "Insufficient output space");
+
+   this->binary_encode(output.data(), output.size());
 }
 
 /*
@@ -400,22 +410,25 @@ void BigInt::binary_encode(uint8_t output[], size_t len) const {
 /*
 * Set this number to the value in buf
 */
-void BigInt::binary_decode(const uint8_t buf[], size_t length) {
+void BigInt::assign_from_bytes(std::span<const uint8_t> bytes) {
    clear();
 
+   const size_t length = bytes.size();
    const size_t full_words = length / sizeof(word);
    const size_t extra_bytes = length % sizeof(word);
 
    secure_vector<word> reg((round_up(full_words + (extra_bytes > 0 ? 1 : 0), 8)));
 
    for(size_t i = 0; i != full_words; ++i) {
-      reg[i] = load_be<word>(buf + length - sizeof(word) * (i + 1), 0);
+      reg[i] = load_be<word>(bytes.last<sizeof(word)>());
+      bytes = bytes.first(bytes.size() - sizeof(word));
    }
 
-   if(extra_bytes > 0) {
-      for(size_t i = 0; i != extra_bytes; ++i) {
-         reg[full_words] = (reg[full_words] << 8) | buf[i];
-      }
+   if(!bytes.empty()) {
+      BOTAN_ASSERT_NOMSG(extra_bytes == bytes.size());
+      std::array<uint8_t, sizeof(word)> last_partial_word = {0};
+      copy_mem(std::span{last_partial_word}.last(extra_bytes), bytes);
+      reg[full_words] = load_be<word>(last_partial_word);
    }
 
    m_data.swap(reg);
@@ -427,20 +440,20 @@ void BigInt::ct_cond_add(bool predicate, const BigInt& value) {
    }
    this->grow_to(1 + value.sig_words());
 
-   bigint_cnd_add(static_cast<word>(predicate), this->mutable_data(), this->size(), value.data(), value.sig_words());
+   bigint_cnd_add(static_cast<word>(predicate), this->mutable_data(), this->size(), value._data(), value.sig_words());
 }
 
 void BigInt::ct_shift_left(size_t shift) {
    auto shl_bit = [](const BigInt& a, BigInt& result) {
       BOTAN_DEBUG_ASSERT(a.size() + 1 == result.size());
-      bigint_shl2(result.mutable_data(), a.data(), a.size(), 1);
+      bigint_shl2(result.mutable_data(), a._data(), a.size(), 1);
       // shl2 may have shifted a bit into the next word, which must be dropped
       clear_mem(result.mutable_data() + result.size() - 1, 1);
    };
 
    auto shl_word = [](const BigInt& a, BigInt& result) {
       // the most significant word is not copied, aka. shifted out
-      bigint_shl2(result.mutable_data(), a.data(), a.size() - 1 /* ignore msw */, BOTAN_MP_WORD_BITS);
+      bigint_shl2(result.mutable_data(), a._data(), a.size() - 1 /* ignore msw */, BOTAN_MP_WORD_BITS);
       // we left-shifted by a full word, the least significant word must be zero'ed
       clear_mem(result.mutable_data(), 1);
    };
