@@ -17,6 +17,7 @@
 #include <botan/internal/mp_asmi.h>
 #include <algorithm>
 #include <array>
+#include <span>
 
 namespace Botan {
 
@@ -137,10 +138,7 @@ inline constexpr void bigint_cnd_add_or_sub(CT::Mask<W> mask, W x[], const W y[]
    for(size_t i = 0; i != blocks; i += 8) {
       carry = word8_add3(t0, x + i, y + i, carry);
       borrow = word8_sub3(t1, x + i, y + i, borrow);
-
-      for(size_t j = 0; j != 8; ++j) {
-         x[i + j] = mask.select(t0[j], t1[j]);
-      }
+      mask.select_n(x + i, t0, t1, 8);
    }
 
    for(size_t i = blocks; i != size; ++i) {
@@ -173,10 +171,7 @@ inline constexpr auto bigint_cnd_addsub(CT::Mask<W> mask, W x[], const W y[], co
    for(size_t i = 0; i != blocks; i += 8) {
       carry = word8_add3(t0, x + i, y + i, carry);
       borrow = word8_sub3(t1, x + i, z + i, borrow);
-
-      for(size_t j = 0; j != 8; ++j) {
-         x[i + j] = mask.select(t0[j], t1[j]);
-      }
+      mask.select_n(x + i, t0, t1, 8);
    }
 
    for(size_t i = blocks; i != size; ++i) {
@@ -258,6 +253,17 @@ inline constexpr auto bigint_add3_nc(W z[], const W x[], size_t x_size, const W
    return carry;
 }
 
+template <WordType W, size_t N>
+inline constexpr auto bigint_add(std::span<W, N> z, std::span<const W, N> x, std::span<const W, N> y) -> W {
+   if constexpr(N == 4) {
+      return word4_add3<W>(z.data(), x.data(), y.data(), 0);
+   } else if constexpr(N == 8) {
+      return word8_add3<W>(z.data(), x.data(), y.data(), 0);
+   } else {
+      return bigint_add3_nc(z.data(), x.data(), N, y.data(), N);
+   }
+}
+
 /**
 * Two operand addition
 * @param x the first operand (and output)
@@ -395,16 +401,21 @@ inline constexpr void bigint_monty_maybe_sub(size_t N, W z[], W x0, const W x[],
 */
 template <size_t N, WordType W>
 inline constexpr void bigint_monty_maybe_sub(W z[N], W x0, const W x[N], const W y[N]) {
-   const constexpr size_t blocks = N - (N % 8);
-
    W borrow = 0;
 
-   for(size_t i = 0; i != blocks; i += 8) {
-      borrow = word8_sub3(z + i, x + i, y + i, borrow);
-   }
+   if constexpr(N == 4) {
+      borrow = word4_sub3(z, x, y, borrow);
+   } else if constexpr(N == 8) {
+      borrow = word8_sub3(z, x, y, borrow);
+   } else {
+      const constexpr size_t blocks = N - (N % 8);
+      for(size_t i = 0; i != blocks; i += 8) {
+         borrow = word8_sub3(z + i, x + i, y + i, borrow);
+      }
 
-   for(size_t i = blocks; i != N; ++i) {
-      z[i] = word_sub(x[i], y[i], &borrow);
+      for(size_t i = blocks; i != N; ++i) {
+         z[i] = word_sub(x[i], y[i], &borrow);
+      }
    }
 
    word_sub(x0, static_cast<W>(0), &borrow);
@@ -851,17 +862,32 @@ inline constexpr W shift_left(std::array<W, N>& x) {
    static_assert(S < WordInfo<W>::bits, "Shift too large");
 
    W carry = 0;
-   for(size_t j = 0; j != N; ++j) {
-      const W w = x[j];
-      x[j] = (w << S) | carry;
+   for(size_t i = 0; i != N; ++i) {
+      const W w = x[i];
+      x[i] = (w << S) | carry;
       carry = w >> (WordInfo<W>::bits - S);
    }
 
    return carry;
 }
 
+template <size_t S, WordType W, size_t N>
+inline constexpr W shift_right(std::array<W, N>& x) {
+   static_assert(S < WordInfo<W>::bits, "Shift too large");
+
+   W carry = 0;
+   for(size_t i = 0; i != N; ++i) {
+      const W w = x[N - 1 - i];
+      x[N - 1 - i] = (w >> S) | carry;
+      carry = w << (WordInfo<W>::bits - S);
+   }
+
+   return carry;
+}
+
+// Should be consteval but this triggers a bug in Clang 14
 template <WordType W, size_t N>
-consteval auto hex_to_words(const char (&s)[N]) {
+constexpr auto hex_to_words(const char (&s)[N]) {
    // Char count includes null terminator which we ignore
    const constexpr size_t C = N - 1;
 
@@ -896,50 +922,12 @@ consteval auto hex_to_words(const char (&s)[N]) {
    return r;
 }
 
-/*
-* Comba Fixed Length Multiplication
-*/
-template <size_t N, WordType W>
-constexpr inline void comba_mul(W z[2 * N], const W x[N], const W y[N]) {
-   W w2 = 0, w1 = 0, w0 = 0;
-
-   for(size_t i = 0; i != 2 * N; ++i) {
-      const size_t start = i + 1 < N ? 0 : i + 1 - N;
-      const size_t end = std::min(N, i + 1);
-
-      for(size_t j = start; j != end; ++j) {
-         word3_muladd(&w2, &w1, &w0, x[j], y[i - j]);
-      }
-      z[i] = w0;
-      w0 = w1;
-      w1 = w2;
-      w2 = 0;
-   }
-}
-
-template <size_t N, WordType W>
-constexpr inline void comba_sqr(W z[2 * N], const W x[N]) {
-   W w2 = 0, w1 = 0, w0 = 0;
-
-   for(size_t i = 0; i != 2 * N; ++i) {
-      const size_t start = i + 1 < N ? 0 : i + 1 - N;
-      const size_t end = std::min(N, i + 1);
-
-      for(size_t j = start; j != end; ++j) {
-         word3_muladd(&w2, &w1, &w0, x[j], x[i - j]);
-      }
-      z[i] = w0;
-      w0 = w1;
-      w1 = w2;
-      w2 = 0;
-   }
-}
-
 /*
 * Comba Multiplication / Squaring
 */
 BOTAN_FUZZER_API void bigint_comba_mul4(word z[8], const word x[4], const word y[4]);
 BOTAN_FUZZER_API void bigint_comba_mul6(word z[12], const word x[6], const word y[6]);
+BOTAN_FUZZER_API void bigint_comba_mul7(word z[14], const word x[7], const word y[7]);
 BOTAN_FUZZER_API void bigint_comba_mul8(word z[16], const word x[8], const word y[8]);
 BOTAN_FUZZER_API void bigint_comba_mul9(word z[18], const word x[9], const word y[9]);
 BOTAN_FUZZER_API void bigint_comba_mul16(word z[32], const word x[16], const word y[16]);
@@ -947,11 +935,87 @@ BOTAN_FUZZER_API void bigint_comba_mul24(word z[48], const word x[24], const wor
 
 BOTAN_FUZZER_API void bigint_comba_sqr4(word out[8], const word in[4]);
 BOTAN_FUZZER_API void bigint_comba_sqr6(word out[12], const word in[6]);
+BOTAN_FUZZER_API void bigint_comba_sqr7(word out[14], const word x[7]);
 BOTAN_FUZZER_API void bigint_comba_sqr8(word out[16], const word in[8]);
 BOTAN_FUZZER_API void bigint_comba_sqr9(word out[18], const word in[9]);
 BOTAN_FUZZER_API void bigint_comba_sqr16(word out[32], const word in[16]);
 BOTAN_FUZZER_API void bigint_comba_sqr24(word out[48], const word in[24]);
 
+/*
+* Comba Fixed Length Multiplication
+*/
+template <size_t N, WordType W>
+constexpr inline void comba_mul(W z[2 * N], const W x[N], const W y[N]) {
+   if(!std::is_constant_evaluated()) {
+      if constexpr(std::same_as<W, word> && N == 4) {
+         return bigint_comba_mul4(z, x, y);
+      }
+      if constexpr(std::same_as<W, word> && N == 6) {
+         return bigint_comba_mul6(z, x, y);
+      }
+      if constexpr(std::same_as<W, word> && N == 7) {
+         return bigint_comba_mul7(z, x, y);
+      }
+      if constexpr(std::same_as<W, word> && N == 8) {
+         return bigint_comba_mul8(z, x, y);
+      }
+      if constexpr(std::same_as<W, word> && N == 9) {
+         return bigint_comba_mul9(z, x, y);
+      }
+      if constexpr(std::same_as<W, word> && N == 16) {
+         return bigint_comba_mul16(z, x, y);
+      }
+   }
+
+   word3<W> accum;
+
+   for(size_t i = 0; i != 2 * N; ++i) {
+      const size_t start = i + 1 < N ? 0 : i + 1 - N;
+      const size_t end = std::min(N, i + 1);
+
+      for(size_t j = start; j != end; ++j) {
+         accum.mul(x[j], y[i - j]);
+      }
+      z[i] = accum.extract();
+   }
+}
+
+template <size_t N, WordType W>
+constexpr inline void comba_sqr(W z[2 * N], const W x[N]) {
+   if(!std::is_constant_evaluated()) {
+      if constexpr(std::same_as<W, word> && N == 4) {
+         return bigint_comba_sqr4(z, x);
+      }
+      if constexpr(std::same_as<W, word> && N == 6) {
+         return bigint_comba_sqr6(z, x);
+      }
+      if constexpr(std::same_as<W, word> && N == 7) {
+         return bigint_comba_sqr7(z, x);
+      }
+      if constexpr(std::same_as<W, word> && N == 8) {
+         return bigint_comba_sqr8(z, x);
+      }
+      if constexpr(std::same_as<W, word> && N == 9) {
+         return bigint_comba_sqr9(z, x);
+      }
+      if constexpr(std::same_as<W, word> && N == 16) {
+         return bigint_comba_sqr16(z, x);
+      }
+   }
+
+   word3<W> accum;
+
+   for(size_t i = 0; i != 2 * N; ++i) {
+      const size_t start = i + 1 < N ? 0 : i + 1 - N;
+      const size_t end = std::min(N, i + 1);
+
+      for(size_t j = start; j != end; ++j) {
+         accum.mul(x[j], x[i - j]);
+      }
+      z[i] = accum.extract();
+   }
+}
+
 /*
 * Montgomery reduction
 *
