@@ -12,6 +12,7 @@
 #include <botan/numthry.h>
 #include <botan/rng.h>
 #include <botan/internal/ct_utils.h>
+#include <botan/internal/stl_util.h>
 
 namespace Botan {
 
@@ -360,14 +361,12 @@ EC_Point& EC_Point::operator*=(const BigInt& scalar) {
    return *this;
 }
 
-EC_Point operator*(const BigInt& scalar, const EC_Point& point) {
-   BOTAN_DEBUG_ASSERT(point.on_the_curve());
-
+EC_Point EC_Point::mul(const BigInt& scalar) const {
    const size_t scalar_bits = scalar.bits();
 
    std::vector<BigInt> ws(EC_Point::WORKSPACE_SIZE);
 
-   EC_Point R[2] = {point.zero(), point};
+   EC_Point R[2] = {this->zero(), *this};
 
    for(size_t i = scalar_bits; i > 0; i--) {
       const size_t b = scalar.get_bit(i - 1);
@@ -385,7 +384,7 @@ EC_Point operator*(const BigInt& scalar, const EC_Point& point) {
 }
 
 //static
-void EC_Point::force_all_affine(std::vector<EC_Point>& points, secure_vector<word>& ws) {
+void EC_Point::force_all_affine(std::span<EC_Point> points, secure_vector<word>& ws) {
    if(points.size() <= 1) {
       for(auto& point : points) {
          point.force_affine();
@@ -466,6 +465,28 @@ bool EC_Point::is_affine() const {
    return m_curve.is_one(m_coord_z);
 }
 
+secure_vector<uint8_t> EC_Point::x_bytes() const {
+   const size_t p_bytes = m_curve.get_p_bytes();
+   secure_vector<uint8_t> b(p_bytes);
+   BigInt::encode_1363(b.data(), b.size(), this->get_affine_x());
+   return b;
+}
+
+secure_vector<uint8_t> EC_Point::y_bytes() const {
+   const size_t p_bytes = m_curve.get_p_bytes();
+   secure_vector<uint8_t> b(p_bytes);
+   BigInt::encode_1363(b.data(), b.size(), this->get_affine_y());
+   return b;
+}
+
+secure_vector<uint8_t> EC_Point::xy_bytes() const {
+   const size_t p_bytes = m_curve.get_p_bytes();
+   secure_vector<uint8_t> b(2 * p_bytes);
+   BigInt::encode_1363(&b[0], p_bytes, this->get_affine_x());
+   BigInt::encode_1363(&b[p_bytes], p_bytes, this->get_affine_y());
+   return b;
+}
+
 BigInt EC_Point::get_affine_x() const {
    if(is_zero()) {
       throw Invalid_State("Cannot convert zero point to affine");
@@ -543,8 +564,8 @@ bool EC_Point::on_the_curve() const {
    return true;
 }
 
-// swaps the states of *this and other, does not throw!
-void EC_Point::swap(EC_Point& other) {
+// swaps the states of *this and other
+void EC_Point::swap(EC_Point& other) noexcept {
    m_curve.swap(other.m_curve);
    m_coord_x.swap(other.m_coord_x);
    m_coord_y.swap(other.m_coord_y);
@@ -575,22 +596,22 @@ std::vector<uint8_t> EC_Point::encode(EC_Point_Format format) const {
    const BigInt x = get_affine_x();
    const BigInt y = get_affine_y();
 
-   std::vector<uint8_t> result;
+   const size_t parts = (format == EC_Point_Format::Compressed) ? 1 : 2;
+
+   std::vector<uint8_t> result(1 + parts * p_bytes);
+   BufferStuffer stuffer(result);
 
    if(format == EC_Point_Format::Uncompressed) {
-      result.resize(1 + 2 * p_bytes);
-      result[0] = 0x04;
-      BigInt::encode_1363(&result[1], p_bytes, x);
-      BigInt::encode_1363(&result[1 + p_bytes], p_bytes, y);
+      stuffer.append(0x04);
+      x.serialize_to(stuffer.next(p_bytes));
+      y.serialize_to(stuffer.next(p_bytes));
    } else if(format == EC_Point_Format::Compressed) {
-      result.resize(1 + p_bytes);
-      result[0] = 0x02 | static_cast<uint8_t>(y.get_bit(0));
-      BigInt::encode_1363(&result[1], p_bytes, x);
+      stuffer.append(0x02 | static_cast<uint8_t>(y.get_bit(0)));
+      x.serialize_to(stuffer.next(p_bytes));
    } else if(format == EC_Point_Format::Hybrid) {
-      result.resize(1 + 2 * p_bytes);
-      result[0] = 0x06 | static_cast<uint8_t>(y.get_bit(0));
-      BigInt::encode_1363(&result[1], p_bytes, x);
-      BigInt::encode_1363(&result[1 + p_bytes], p_bytes, y);
+      stuffer.append(0x06 | static_cast<uint8_t>(y.get_bit(0)));
+      x.serialize_to(stuffer.next(p_bytes));
+      y.serialize_to(stuffer.next(p_bytes));
    } else {
       throw Invalid_Argument("EC2OSP illegal point encoding");
    }
@@ -682,4 +703,8 @@ std::pair<BigInt, BigInt> OS2ECP(
    return std::make_pair(x, y);
 }
 
+EC_Point OS2ECP(std::span<const uint8_t> data, const CurveGFp& curve) {
+   return OS2ECP(data.data(), data.size(), curve);
+}
+
 }  // namespace Botan
