@@ -18,6 +18,7 @@
 #include <botan/internal/pk_ops_impl.h>
 #include <botan/internal/point_mul.h>
 #include <botan/internal/scan_name.h>
+#include <botan/internal/stl_util.h>
 
 namespace Botan {
 
@@ -72,13 +73,8 @@ std::unique_ptr<HashFunction> eckcdsa_signature_hash(const AlgorithmIdentifier&
    return HashFunction::create_or_throw(oid_info[1]);
 }
 
-std::vector<uint8_t> eckcdsa_prefix(const PointGFp& point, size_t order_bytes, size_t hash_block_size) {
-   const BigInt public_x = point.get_affine_x();
-   const BigInt public_y = point.get_affine_y();
-
-   std::vector<uint8_t> prefix(2 * order_bytes);
-   BigInt::encode_1363(&prefix[0], order_bytes, public_x);
-   BigInt::encode_1363(&prefix[order_bytes], order_bytes, public_y);
+std::vector<uint8_t> eckcdsa_prefix(const EC_Point& point, size_t hash_block_size) {
+   auto prefix = concat<std::vector<uint8_t>>(point.x_bytes(), point.y_bytes());
 
    // Either truncate or zero-extend to match the hash block size
    prefix.resize(hash_block_size);
@@ -126,7 +122,7 @@ class ECKCDSA_Signature_Operation final : public PK_Ops::Signature {
             m_x(eckcdsa.private_value()),
             m_hash(eckcdsa_signature_hash(padding)),
             m_prefix_used(false) {
-         m_prefix = eckcdsa_prefix(eckcdsa.public_point(), m_group.get_order_bytes(), m_hash->hash_block_size());
+         m_prefix = eckcdsa_prefix(eckcdsa.public_point(), m_hash->hash_block_size());
       }
 
       void update(const uint8_t msg[], size_t msg_len) override {
@@ -173,29 +169,23 @@ secure_vector<uint8_t> ECKCDSA_Signature_Operation::raw_sign(const uint8_t msg[]
    const BigInt k = m_group.random_scalar(rng);
    const BigInt k_times_P_x = m_group.blinded_base_point_multiply_x(k, rng, m_ws);
 
-   secure_vector<uint8_t> to_be_hashed(k_times_P_x.bytes());
-   k_times_P_x.binary_encode(to_be_hashed.data());
-
    auto hash = m_hash->new_object();
-   hash->update(to_be_hashed);
+   hash->update(k_times_P_x.serialize(m_group.get_order_bytes()));
    secure_vector<uint8_t> c = hash->final();
    truncate_hash_if_needed(c, m_group.get_order_bytes());
 
-   const BigInt r(c.data(), c.size());
+   const auto r = c;
 
    BOTAN_ASSERT_NOMSG(msg_len == c.size());
    xor_buf(c, msg, c.size());
-   BigInt w(c.data(), c.size());
-   w = m_group.mod_order(w);
+   const BigInt w = m_group.mod_order(BigInt::from_bytes(c));
 
    const BigInt s = m_group.multiply_mod_order(m_x, k - w);
    if(s.is_zero()) {
       throw Internal_Error("During ECKCDSA signature generation created zero s");
    }
 
-   secure_vector<uint8_t> output = BigInt::encode_1363(r, c.size());
-   output += BigInt::encode_1363(s, m_group.get_order_bytes());
-   return output;
+   return concat(r, s.serialize(m_group.get_order_bytes()));
 }
 
 /**
@@ -208,7 +198,7 @@ class ECKCDSA_Verification_Operation final : public PK_Ops::Verification {
             m_gy_mul(m_group.get_base_point(), eckcdsa.public_point()),
             m_hash(eckcdsa_signature_hash(padding)),
             m_prefix_used(false) {
-         m_prefix = eckcdsa_prefix(eckcdsa.public_point(), m_group.get_order_bytes(), m_hash->hash_block_size());
+         m_prefix = eckcdsa_prefix(eckcdsa.public_point(), m_hash->hash_block_size());
       }
 
       ECKCDSA_Verification_Operation(const ECKCDSA_PublicKey& eckcdsa, const AlgorithmIdentifier& alg_id) :
@@ -216,7 +206,7 @@ class ECKCDSA_Verification_Operation final : public PK_Ops::Verification {
             m_gy_mul(m_group.get_base_point(), eckcdsa.public_point()),
             m_hash(eckcdsa_signature_hash(alg_id)),
             m_prefix_used(false) {
-         m_prefix = eckcdsa_prefix(eckcdsa.public_point(), m_group.get_order_bytes(), m_hash->hash_block_size());
+         m_prefix = eckcdsa_prefix(eckcdsa.public_point(), m_hash->hash_block_size());
       }
 
       void update(const uint8_t msg[], size_t msg_len) override;
@@ -271,19 +261,16 @@ bool ECKCDSA_Verification_Operation::verify(const uint8_t msg[], size_t msg_len,
 
    secure_vector<uint8_t> r_xor_e(r);
    xor_buf(r_xor_e, msg, r.size());
-   BigInt w(r_xor_e.data(), r_xor_e.size());
-   w = m_group.mod_order(w);
+
+   const BigInt w = m_group.mod_order(BigInt::from_bytes(r_xor_e));
 
    const EC_Point q = m_gy_mul.multi_exp(w, s);
    if(q.is_zero()) {
       return false;
    }
 
-   const BigInt q_x = q.get_affine_x();
-   secure_vector<uint8_t> c(q_x.bytes());
-   q_x.binary_encode(c.data());
    auto c_hash = m_hash->new_object();
-   c_hash->update(c.data(), c.size());
+   c_hash->update(q.x_bytes());
    secure_vector<uint8_t> v = c_hash->final();
    truncate_hash_if_needed(v, m_group.get_order_bytes());
 
