@@ -27,13 +27,19 @@ std::vector<std::pair<std::string, std::string>> algorithm_specs_for_group(Group
    switch(group.code()) {
       case Group_Params::HYBRID_X25519_KYBER_512_R3_OQS:
       case Group_Params::HYBRID_X25519_KYBER_512_R3_CLOUDFLARE:
-         return {{"Curve25519", "Curve25519"}, {"Kyber", "Kyber-512-r3"}};
+         return {{"X25519", "X25519"}, {"Kyber", "Kyber-512-r3"}};
       case Group_Params::HYBRID_X25519_KYBER_768_R3_OQS:
-         return {{"Curve25519", "Curve25519"}, {"Kyber", "Kyber-768-r3"}};
+         return {{"X25519", "X25519"}, {"Kyber", "Kyber-768-r3"}};
+      case Group_Params::HYBRID_X448_KYBER_768_R3_OQS:
+         return {{"X448", "X448"}, {"Kyber", "Kyber-768-r3"}};
       case Group_Params::HYBRID_X25519_eFRODOKEM_640_SHAKE_OQS:
-         return {{"Curve25519", "Curve25519"}, {"FrodoKEM", "eFrodoKEM-640-SHAKE"}};
+         return {{"X25519", "X25519"}, {"FrodoKEM", "eFrodoKEM-640-SHAKE"}};
       case Group_Params::HYBRID_X25519_eFRODOKEM_640_AES_OQS:
-         return {{"Curve25519", "Curve25519"}, {"FrodoKEM", "eFrodoKEM-640-AES"}};
+         return {{"X25519", "X25519"}, {"FrodoKEM", "eFrodoKEM-640-AES"}};
+      case Group_Params::HYBRID_X448_eFRODOKEM_976_SHAKE_OQS:
+         return {{"X448", "X448"}, {"FrodoKEM", "eFrodoKEM-976-SHAKE"}};
+      case Group_Params::HYBRID_X448_eFRODOKEM_976_AES_OQS:
+         return {{"X448", "X448"}, {"FrodoKEM", "eFrodoKEM-976-AES"}};
 
       case Group_Params::HYBRID_SECP256R1_KYBER_512_R3_OQS:
          return {{"ECDH", "secp256r1"}, {"Kyber", "Kyber-512-r3"}};
@@ -97,10 +103,16 @@ std::vector<size_t> public_value_lengths_for_group(Group_Params group) {
          return {32, 800};
       case Group_Params::HYBRID_X25519_KYBER_768_R3_OQS:
          return {32, 1184};
+      case Group_Params::HYBRID_X448_KYBER_768_R3_OQS:
+         return {56, 1184};
       case Group_Params::HYBRID_X25519_eFRODOKEM_640_SHAKE_OQS:
          return {32, 9616};
       case Group_Params::HYBRID_X25519_eFRODOKEM_640_AES_OQS:
          return {32, 9616};
+      case Group_Params::HYBRID_X448_eFRODOKEM_976_SHAKE_OQS:
+         return {56, 15632};
+      case Group_Params::HYBRID_X448_eFRODOKEM_976_AES_OQS:
+         return {56, 15632};
 
       case Group_Params::HYBRID_SECP256R1_KYBER_512_R3_OQS:
          return {32, 800};
@@ -209,15 +221,10 @@ AlgorithmIdentifier Hybrid_KEM_PublicKey::algorithm_identifier() const {
 }
 
 std::vector<uint8_t> Hybrid_KEM_PublicKey::public_key_bits() const {
-   // Technically, that's not really correct. The docstring for public_key_bits()
-   // states that it should return a BER-encoding of the public key.
-   //
-   // TODO: Perhaps add something like Public_Key::raw_public_key_bits() to
-   //       better reflect what we need here.
-   return public_value();
+   return raw_public_key_bits();
 }
 
-std::vector<uint8_t> Hybrid_KEM_PublicKey::public_value() const {
+std::vector<uint8_t> Hybrid_KEM_PublicKey::raw_public_key_bits() const {
    // draft-ietf-tls-hybrid-design-06 3.2
    //   The values are directly concatenated, without any additional encoding
    //   or length fields; this assumes that the representation and length of
@@ -226,15 +233,7 @@ std::vector<uint8_t> Hybrid_KEM_PublicKey::public_value() const {
    //   other unambiguous encoding must be used to ensure that the composition
    //   of the two values is injective.
    return reduce(m_public_keys, std::vector<uint8_t>(), [](auto pkb, const auto& key) {
-      // Technically, this is not correct! `public_key_bits()` is meant to
-      // return a BER-encoded public key. For (e.g.) Kyber, that contract is
-      // broken: It returns the raw encoding as used in the reference
-      // implementation.
-      //
-      // TODO: Provide something like Public_Key::raw_public_key_bits() to
-      //       reflect that difference. Also: Key agreement keys could return
-      //       their raw public value there.
-      return concat(pkb, key->public_key_bits());
+      return concat(pkb, key->raw_public_key_bits());
    });
 }
 
