@@ -22,8 +22,8 @@
 #include <botan/internal/ct_utils.h>
 #include <botan/internal/stl_util.h>
 
-#if defined(BOTAN_HAS_CURVE_25519)
-   #include <botan/curve25519.h>
+#if defined(BOTAN_HAS_X25519)
+   #include <botan/x25519.h>
 #endif
 
 #if defined(BOTAN_HAS_X448)
@@ -99,7 +99,7 @@ void TLS::Callbacks::tls_verify_cert_chain(const std::vector<X509_Certificate>&
    Path_Validation_Result result = x509_path_validate(cert_chain,
                                                       restrictions,
                                                       trusted_roots,
-                                                      (usage == Usage_Type::TLS_SERVER_AUTH ? hostname : ""),
+                                                      hostname,
                                                       usage,
                                                       tls_current_timestamp(),
                                                       tls_verify_cert_chain_ocsp_timeout(),
@@ -268,11 +268,11 @@ std::unique_ptr<PK_Key_Agreement_Key> TLS::Callbacks::tls_generate_ephemeral_key
    const auto group_params = std::get<TLS::Group_Params>(group);
 
    if(group_params.is_ecdh_named_curve()) {
-      const EC_Group ec_group(group_params.to_string().value());
+      const auto ec_group = EC_Group::from_name(group_params.to_string().value());
       return std::make_unique<ECDH_PrivateKey>(rng, ec_group);
    }
 
-#if defined(BOTAN_HAS_CURVE_25519)
+#if defined(BOTAN_HAS_X25519)
    if(group_params.is_x25519()) {
       return std::make_unique<X25519_PrivateKey>(rng);
    }
@@ -307,7 +307,7 @@ secure_vector<uint8_t> TLS::Callbacks::tls_ephemeral_key_agreement(
       // a standardized DH group identifier.
       const auto dl_group = get_dl_group(group);
 
-      auto Y = BigInt::decode(public_value);
+      auto Y = BigInt::from_bytes(public_value);
 
       /*
        * A basic check for key validity. As we do not know q here we
@@ -329,20 +329,20 @@ secure_vector<uint8_t> TLS::Callbacks::tls_ephemeral_key_agreement(
    const auto group_params = std::get<TLS::Group_Params>(group);
 
    if(group_params.is_ecdh_named_curve()) {
-      const EC_Group ec_group(group_params.to_string().value());
+      const auto ec_group = EC_Group::from_name(group_params.to_string().value());
       ECDH_PublicKey peer_key(ec_group, ec_group.OS2ECP(public_value));
       policy.check_peer_key_acceptable(peer_key);
 
       return agree(private_key, peer_key);
    }
 
-#if defined(BOTAN_HAS_CURVE_25519)
+#if defined(BOTAN_HAS_X25519)
    if(group_params.is_x25519()) {
       if(public_value.size() != 32) {
          throw TLS_Exception(Alert::HandshakeFailure, "Invalid X25519 key size");
       }
 
-      Curve25519_PublicKey peer_key(public_value);
+      X25519_PublicKey peer_key(public_value);
       policy.check_peer_key_acceptable(peer_key);
 
       return agree(private_key, peer_key);
