@@ -128,10 +128,6 @@ enum class Endianness : bool {
    Little,
 };
 
-struct AutoDetect {
-      constexpr AutoDetect() = delete;
-};
-
 /**
  * @warning This function may return false if the native endianness is unknown
  * @returns true iff the native endianness matches the given endianness
@@ -359,8 +355,8 @@ inline constexpr void load_any(OutR&& out, InR&& in) {
  */
 template <Endianness endianness, typename OutT, ranges::contiguous_range<uint8_t> InR>
    requires(std::same_as<AutoDetect, OutT> ||
-            ((ranges::statically_spanable_range<OutT> ||
-              concepts::resizable_container<OutT>)&&unsigned_integralish<typename OutT::value_type>))
+            ((ranges::statically_spanable_range<OutT> || concepts::resizable_container<OutT>) &&
+             unsigned_integralish<typename OutT::value_type>))
 inline constexpr auto load_any(InR&& in_range) {
    auto out = []([[maybe_unused]] const auto& in) {
       if constexpr(std::same_as<AutoDetect, OutT>) {
@@ -715,7 +711,7 @@ inline constexpr auto store_be(ParamTs&&... params) {
 namespace detail {
 
 template <Endianness endianness, unsigned_integralish T>
-size_t copy_out_any_word_aligned_portion(std::span<uint8_t>& out, std::span<const T>& in) {
+inline size_t copy_out_any_word_aligned_portion(std::span<uint8_t>& out, std::span<const T>& in) {
    const size_t full_words = out.size() / sizeof(T);
    const size_t full_word_bytes = full_words * sizeof(T);
    const size_t remaining_bytes = out.size() - full_word_bytes;
@@ -736,7 +732,7 @@ size_t copy_out_any_word_aligned_portion(std::span<uint8_t>& out, std::span<cons
  * byte order.
  */
 template <ranges::spanable_range InR>
-void copy_out_be(std::span<uint8_t> out, InR&& in) {
+inline void copy_out_be(std::span<uint8_t> out, InR&& in) {
    using T = std::ranges::range_value_t<InR>;
    std::span<const T> in_s{in};
    const auto remaining_bytes = detail::copy_out_any_word_aligned_portion<detail::Endianness::Big>(out, in_s);
@@ -752,7 +748,7 @@ void copy_out_be(std::span<uint8_t> out, InR&& in) {
  * byte order.
  */
 template <ranges::spanable_range InR>
-void copy_out_le(std::span<uint8_t> out, InR&& in) {
+inline void copy_out_le(std::span<uint8_t> out, InR&& in) {
    using T = std::ranges::range_value_t<InR>;
    std::span<const T> in_s{in};
    const auto remaining_bytes = detail::copy_out_any_word_aligned_portion<detail::Endianness::Little>(out, in_s);
