@@ -15,6 +15,8 @@
 #include <algorithm>
 #include <chrono>
 #include <cstdlib>
+#include <iomanip>
+#include <sstream>
 
 #if defined(BOTAN_TARGET_OS_HAS_EXPLICIT_BZERO)
    #include <string.h>
@@ -325,6 +327,26 @@ uint64_t OS::get_system_timestamp_ns() {
    return std::chrono::duration_cast<std::chrono::nanoseconds>(now).count();
 }
 
+std::string OS::format_time(time_t time, const std::string& format) {
+   std::tm tm;
+
+#if defined(BOTAN_TARGET_OS_HAS_WIN32)
+   localtime_s(&tm, &time);
+#elif defined(BOTAN_TARGET_OS_HAS_POSIX1)
+   localtime_r(&time, &tm);
+#else
+   if(auto tmp = std::localtime(&time)) {
+      tm = *tmp;
+   } else {
+      throw Encoding_Error("Could not convert time_t to localtime");
+   }
+#endif
+
+   std::ostringstream oss;
+   oss << std::put_time(&tm, format.c_str());
+   return oss.str();
+}
+
 size_t OS::system_page_size() {
    const size_t default_page_size = 4096;
 
