@@ -65,6 +65,7 @@ struct X509_Certificate_Data {
       bool m_self_signed = false;
       bool m_is_ca_certificate = false;
       bool m_serial_negative = false;
+      bool m_subject_alt_name_exists = false;
 };
 
 std::string X509_Certificate::PEM_label() const {
@@ -130,13 +131,12 @@ std::unique_ptr<X509_Certificate_Data> parse_x509_cert_body(const X509_Object& o
 
    public_key.assert_is_a(ASN1_Type::Sequence, ASN1_Class::Constructed, "X.509 certificate public key");
 
-   // crude method to save the serial's sign; will get lost during decoding, otherwise
-   data->m_serial_negative = serial_bn.is_negative();
-
    // for general sanity convert wire version (0 based) to standards version (v1 .. v3)
    data->m_version += 1;
 
-   data->m_serial = BigInt::encode(serial_bn);
+   data->m_serial = serial_bn.serialize();
+   // crude method to save the serial's sign; will get lost during decoding, otherwise
+   data->m_serial_negative = serial_bn.is_negative();
    data->m_subject_dn_bits = ASN1::put_in_sequence(data->m_subject_dn.get_bits());
    data->m_issuer_dn_bits = ASN1::put_in_sequence(data->m_issuer_dn.get_bits());
 
@@ -179,6 +179,10 @@ std::unique_ptr<X509_Certificate_Data> parse_x509_cert_body(const X509_Object& o
       data->m_name_constraints = ext->get_name_constraints();
    }
 
+   if(auto ext = data->m_v3_extensions.get_extension_object_as<Cert_Extension::Extended_Key_Usage>()) {
+      data->m_extended_key_usage = ext->object_identifiers();
+   }
+
    if(auto ext = data->m_v3_extensions.get_extension_object_as<Cert_Extension::Basic_Constraints>()) {
       if(ext->get_is_ca() == true) {
          /*
@@ -188,7 +192,36 @@ std::unique_ptr<X509_Certificate_Data> parse_x509_cert_body(const X509_Object& o
          * removing this entirely, or alternately adding a warning level
          * validation failure for it.
          */
-         if(data->m_key_constraints.empty() || data->m_key_constraints.includes(Key_Constraints::KeyCertSign)) {
+         const bool allowed_by_ku =
+            data->m_key_constraints.includes(Key_Constraints::KeyCertSign) || data->m_key_constraints.empty();
+
+         /*
+         * If the extended key usages are set then we must restrict the
+         * usage in accordance with it as well.
+         *
+         * RFC 5280 does not define any extended key usages compatible
+         * with certificate signing, but some CAs seem to use serverAuth
+         * or clientAuth here.
+         */
+         const bool allowed_by_ext_ku = [](const std::vector<OID>& ext_ku) -> bool {
+            if(ext_ku.empty()) {
+               return true;
+            }
+
+            const auto server_auth = OID::from_name("PKIX.ServerAuth");
+            const auto client_auth = OID::from_name("PKIX.ClientAuth");
+            const auto ocsp_sign = OID::from_name("PKIX.OCSPSigning");
+
+            for(const auto& oid : ext_ku) {
+               if(oid == server_auth || oid == client_auth || oid == ocsp_sign) {
+                  return true;
+               }
+            }
+
+            return false;
+         }(data->m_extended_key_usage);
+
+         if(allowed_by_ku && allowed_by_ext_ku) {
             data->m_is_ca_certificate = true;
             data->m_path_len_constraint = ext->get_path_limit();
          }
@@ -203,9 +236,11 @@ std::unique_ptr<X509_Certificate_Data> parse_x509_cert_body(const X509_Object& o
       data->m_subject_alt_name = ext->get_alt_name();
    }
 
-   if(auto ext = data->m_v3_extensions.get_extension_object_as<Cert_Extension::Extended_Key_Usage>()) {
-      data->m_extended_key_usage = ext->object_identifiers();
-   }
+   // This will be set even if SAN parsing failed entirely eg due to a decoding error
+   // or if the SAN is empty. This is used to guard against using the CN for domain
+   // name checking.
+   const auto san_oid = OID::from_string("X509v3.SubjectAlternativeName");
+   data->m_subject_alt_name_exists = data->m_v3_extensions.extension_set(san_oid);
 
    if(auto ext = data->m_v3_extensions.get_extension_object_as<Cert_Extension::Certificate_Policies>()) {
       data->m_cert_policies = ext->get_policy_oids();
@@ -251,8 +286,7 @@ std::unique_ptr<X509_Certificate_Data> parse_x509_cert_body(const X509_Object& o
 
    const std::vector<uint8_t> full_encoding = obj.BER_encode();
 
-   auto sha1 = HashFunction::create("SHA-1");
-   if(sha1) {
+   if(auto sha1 = HashFunction::create("SHA-1")) {
       sha1->update(data->m_subject_public_key_bitstring);
       data->m_subject_public_key_bitstring_sha1 = sha1->final_stdvec();
       // otherwise left as empty, and we will throw if subject_public_key_bitstring_sha1 is called
@@ -260,8 +294,7 @@ std::unique_ptr<X509_Certificate_Data> parse_x509_cert_body(const X509_Object& o
       data->m_fingerprint_sha1 = create_hex_fingerprint(full_encoding, "SHA-1");
    }
 
-   auto sha256 = HashFunction::create("SHA-256");
-   if(sha256) {
+   if(auto sha256 = HashFunction::create("SHA-256")) {
       sha256->update(data->m_issuer_dn_bits);
       data->m_issuer_dn_bits_sha256 = sha256->final_stdvec();
 
@@ -490,6 +523,10 @@ std::vector<std::string> X509_Certificate::ca_issuers() const {
    return data().m_ca_issuers;
 }
 
+std::vector<std::string> X509_Certificate::crl_distribution_points() const {
+   return data().m_crl_distribution_points;
+}
+
 std::string X509_Certificate::crl_distribution_point() const {
    // just returns the first (arbitrarily)
    if(!data().m_crl_distribution_points.empty()) {
@@ -506,38 +543,44 @@ const AlternativeName& X509_Certificate::issuer_alt_name() const {
    return data().m_issuer_alt_name;
 }
 
+namespace {
+
+std::vector<std::string> get_cert_user_info(std::string_view req, const X509_DN& dn, const AlternativeName& alt_name) {
+   auto set_to_vector = [](const std::set<std::string>& s) -> std::vector<std::string> { return {s.begin(), s.end()}; };
+
+   if(dn.has_field(req)) {
+      return dn.get_attribute(req);
+   } else if(req == "RFC822" || req == "Email") {
+      return set_to_vector(alt_name.email());
+   } else if(req == "DNS") {
+      return set_to_vector(alt_name.dns());
+   } else if(req == "URI") {
+      return set_to_vector(alt_name.uris());
+   } else if(req == "IP") {
+      std::vector<std::string> ip_str;
+      for(uint32_t ipv4 : alt_name.ipv4_address()) {
+         ip_str.push_back(ipv4_to_string(ipv4));
+      }
+      return ip_str;
+   } else {
+      return {};
+   }
+}
+
+}  // namespace
+
 /*
 * Return information about the subject
 */
 std::vector<std::string> X509_Certificate::subject_info(std::string_view req) const {
-   if(req == "Email") {
-      return this->subject_info("RFC822");
-   }
-
-   if(subject_dn().has_field(req)) {
-      return subject_dn().get_attribute(req);
-   }
-
-   if(subject_alt_name().has_field(req)) {
-      return subject_alt_name().get_attribute(req);
-   }
-
-   return {};
+   return get_cert_user_info(req, subject_dn(), subject_alt_name());
 }
 
 /*
 * Return information about the issuer
 */
 std::vector<std::string> X509_Certificate::issuer_info(std::string_view req) const {
-   if(issuer_dn().has_field(req)) {
-      return issuer_dn().get_attribute(req);
-   }
-
-   if(issuer_alt_name().has_field(req)) {
-      return issuer_alt_name().get_attribute(req);
-   }
-
-   return {};
+   return get_cert_user_info(req, issuer_dn(), issuer_alt_name());
 }
 
 /*
@@ -593,16 +636,21 @@ bool X509_Certificate::matches_dns_name(std::string_view name) const {
       return false;
    }
 
-   auto issued_names = subject_info("DNS");
+   if(auto req_ipv4 = string_to_ipv4(name)) {
+      const auto& ipv4_names = subject_alt_name().ipv4_address();
+      return ipv4_names.contains(req_ipv4.value());
+   } else {
+      auto issued_names = subject_info("DNS");
 
-   // Fall back to CN only if no DNS names are set (RFC 6125 sec 6.4.4)
-   if(issued_names.empty()) {
-      issued_names = subject_info("Name");
-   }
+      // Fall back to CN only if no SAN is included
+      if(!data().m_subject_alt_name_exists) {
+         issued_names = subject_info("Name");
+      }
 
-   for(const auto& issued_name : issued_names) {
-      if(host_wildcard_match(issued_name, name)) {
-         return true;
+      for(const auto& issued_name : issued_names) {
+         if(host_wildcard_match(issued_name, name)) {
+            return true;
+         }
       }
    }
 
@@ -728,8 +776,8 @@ std::string X509_Certificate::to_string() const {
       }
    }
 
-   if(!crl_distribution_point().empty()) {
-      out << "CRL " << crl_distribution_point() << "\n";
+   for(const auto& cdp : crl_distribution_points()) {
+      out << "CRL " << cdp << "\n";
    }
 
    out << "Signature algorithm: " << this->signature_algorithm().oid().to_formatted_string() << "\n";
