@@ -18,26 +18,29 @@ SCRIPT_LOCATION=$(cd "$(dirname "$0")"; pwd)
 
 if type -p "apt-get"; then
 
-    # Hack to deal with https://github.com/actions/runner-images/issues/8659
-    sudo rm -f /etc/apt/sources.list.d/ubuntu-toolchain-r-ubuntu-test-jammy.list
-    sudo apt-get update
-    sudo apt-get install -y --allow-downgrades libc6=2.35-* libc6-dev=2.35-* libstdc++6=12.3.0-* libgcc-s1=12.3.0-*
+    if [ "$(lsb_release -sr)" = "22.04" ]; then
+        # Hack to deal with https://github.com/actions/runner-images/issues/8659
+        sudo rm -f /etc/apt/sources.list.d/ubuntu-toolchain-r-ubuntu-test-jammy.list
+        sudo apt-get update
+        sudo apt-get install -y --allow-downgrades libc6=2.35-* libc6-dev=2.35-* libstdc++6=12.3.0-* libgcc-s1=12.3.0-*
+    fi
 
     # Normal workflow follows
     #sudo apt-get -qq update
-    sudo apt-get -qq install ccache
+    sudo apt-get -qq install ccache libbz2-dev liblzma-dev libsqlite3-dev
 
     if [ "$TARGET" = "valgrind" ] || [ "$TARGET" = "valgrind-full" ]; then
-        sudo apt-get -qq install valgrind
+        # (l)ist mode (avoiding https://github.com/actions/runner-images/issues/9996)
+        sudo NEEDRESTART_MODE=l apt-get -qq install valgrind
 
-    elif [ "$TARGET" = "shared" ] || [ "$TARGET" = "examples" ] || [ "$TARGET" = "tlsanvil" ] ; then
+    elif [ "$TARGET" = "shared" ] || [ "$TARGET" = "examples" ] || [ "$TARGET" = "tlsanvil" ] || [ "$TARGET" = "clang-tidy" ] ; then
         sudo apt-get -qq install libboost-dev
 
     elif [ "$TARGET" = "clang" ]; then
         sudo apt-get -qq install clang
 
     elif [ "$TARGET" = "cross-i386" ]; then
-        sudo apt-get -qq install g++-multilib linux-libc-dev libc6-dev-i386
+        sudo NEEDRESTART_MODE=l apt-get -qq install g++-multilib linux-libc-dev libc6-dev-i386
 
     elif [ "$TARGET" = "cross-win64" ]; then
         sudo apt-get -qq install wine-development g++-mingw-w64-x86-64
@@ -54,6 +57,9 @@ if type -p "apt-get"; then
     elif [ "$TARGET" = "cross-riscv64" ]; then
         sudo apt-get -qq install qemu-user g++-riscv64-linux-gnu
 
+    elif [ "$TARGET" = "cross-mips64" ]; then
+        sudo apt-get -qq install qemu-user g++-mips64-linux-gnuabi64
+
     elif [ "$TARGET" = "cross-s390x" ]; then
         sudo apt-get -qq install qemu-user g++-s390x-linux-gnu
 
@@ -73,6 +79,10 @@ if type -p "apt-get"; then
     elif [ "$TARGET" = "lint" ]; then
         sudo apt-get -qq install pylint python3-matplotlib
 
+    elif [ "$TARGET" = "limbo" ]; then
+        sudo apt-get -qq install python3-dateutil
+        wget -nv https://raw.githubusercontent.com/C2SP/x509-limbo/bd88042508ccfde351b2fee293aebda8971fbebb/limbo.json -O "${SCRIPT_LOCATION}/../../../limbo.json"
+
     elif [ "$TARGET" = "coverage" ] || [ "$TARGET" = "sanitizer" ]; then
         if [ "$TARGET" = "coverage" ]; then
             sudo apt-get -qq install lcov python3-coverage
@@ -92,8 +102,6 @@ if type -p "apt-get"; then
     elif [ "$TARGET" = "docs" ]; then
         sudo apt-get -qq install doxygen python3-docutils python3-sphinx
 
-    elif [ "$TARGET" = "format" ]; then
-        sudo apt-get -qq install clang-format-15
     fi
 else
     export HOMEBREW_NO_AUTO_UPDATE=1
@@ -111,7 +119,11 @@ else
         brew install emscripten
     fi
 
-    sudo xcrun xcode-select --switch '/Applications/Xcode_15.0.app/Contents/Developer'
+    if [ -d '/Applications/Xcode_15.3.app/Contents/Developer' ]; then
+        sudo xcrun xcode-select --switch '/Applications/Xcode_15.4.app/Contents/Developer'
+    else
+        sudo xcrun xcode-select --switch '/Applications/Xcode_15.2.app/Contents/Developer'
+    fi
 fi
 
 # find the ccache cache location and store it in the build job's environment
