@@ -55,6 +55,7 @@ def known_targets():
         'fuzzers',
         'hybrid-tls13-interop-test',
         'lint',
+        'limbo',
         'minimized',
         'nist',
         'sanitizer',
@@ -104,6 +105,8 @@ def build_targets(target, target_os):
 
     if target not in ['examples']:
         yield 'cli'
+
+    if target not in ['examples', 'limbo']:
         yield 'tests'
 
     if target in ['coverage']:
@@ -163,16 +166,18 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache,
         test_cmd += ['--report-properties=%s' %
                      ','.join(['%s:%s' % (sanitize_kv(k), sanitize_kv(v)) for k, v in report_props.items()])]
 
-    install_prefix = tempfile.mkdtemp(prefix='botan-install-')
 
-    flags = ['--prefix=%s' % (install_prefix),
-             '--cc=%s' % (target_cc),
+    flags = ['--cc=%s' % (target_cc),
              '--os=%s' % (target_os),
              '--build-targets=%s' % ','.join(build_targets(target, target_os)),
              '--with-build-dir=%s' % build_dir,
              '--link-method=symlink',
              '--enable-experimental-features']
 
+    if target in ['shared', 'static']:
+        install_prefix = tempfile.mkdtemp(prefix='botan-install-')
+        flags += ['--prefix=%s' % (install_prefix)]
+
     if ccache is not None:
         flags += ['--no-store-vc-rev', '--compiler-cache=%s' % (ccache)]
 
@@ -185,6 +190,10 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache,
     for flag in extra_cxxflags:
         flags += ['--extra-cxxflags=%s' % (flag)]
 
+    if target_os == 'windows':
+        # Workaround for https://github.com/actions/runner-images/issues/10004
+        flags += ['--extra-cxxflags=/D_DISABLE_CONSTEXPR_MUTEX_CONSTRUCTOR']
+
     if target in ['minimized']:
         flags += ['--minimized-build', '--enable-modules=system_rng,sha2_32,sha2_64,aes']
 
@@ -195,15 +204,14 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache,
         # tls is optional for bsi/nist but add it so verify tests work with these minimized configs
         flags += ['--module-policy=%s' % (target), '--enable-modules=tls12']
 
-    if target == 'docs':
+    if target in ['docs']:
         flags += ['--with-doxygen', '--with-sphinx', '--with-rst2man']
+
+    if target in ['docs', 'codeql', 'hybrid-tls13-interop-test', 'limbo']:
         test_cmd = None
 
-    if target == 'codeql':
-        test_cmd = None
-
-    if target == 'hybrid-tls13-interop-test':
-        test_cmd = None
+    if target in ['codeql']:
+        flags += ['--no-optimizations']
 
     if target == 'cross-win64':
         # this test compiles under MinGW but fails when run under Wine
@@ -224,7 +232,14 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache,
 
     if target in ['valgrind', 'valgrind-full']:
         flags += ['--with-valgrind']
-        test_prefix = ['valgrind', '--error-exitcode=9', '-v', '--leak-check=full', '--show-reachable=yes']
+
+        test_prefix = ['valgrind',
+                       '-v',
+                       '--error-exitcode=9',
+                       '--leak-check=full',
+                       '--show-reachable=yes',
+                       '--track-origins=yes']
+
         # valgrind is single threaded anyway
         test_cmd += ['--test-threads=1']
 
@@ -235,11 +250,13 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache,
                 'dh_invalid', 'dh_kat', 'dh_keygen', 'dl_group_gen', 'dlies',
                 'dsa_kat_verify', 'dsa_param', 'ecc_basemul', 'ecdsa_verify_wycheproof',
                 'ed25519_sign', 'elgamal_decrypt', 'elgamal_encrypt', 'elgamal_keygen',
-                'ffi_dsa', 'ffi_elgamal', 'hash_nist_mc', 'mce_keygen',
-                'passhash9', 'pbkdf', 'pwdhash', 'rsa_encrypt', 'rsa_pss', 'rsa_pss_raw',
-                'scrypt', 'srp6_kat', 'srp6_rt', 'unit_tls', 'x509_path_bsi',
-                'x509_path_rsa_pss', 'xmss_keygen', 'xmss_keygen_reference',
-                'xmss_sign', 'xmss_verify', 'xmss_verify_invalid' ]
+                'ffi_dh', 'ffi_dsa', 'ffi_elgamal', 'frodo_kat_tests', 'hash_nist_mc',
+                'hss_lms_keygen', 'hss_lms_sign', 'mce_keygen', 'passhash9', 'pbkdf',
+                'pwdhash', 'rsa_encrypt', 'rsa_pss', 'rsa_pss_raw', 'scrypt',
+                'sphincsplus', 'sphincsplus_fors', 'sphincsplus_keygen', 'srp6_kat',
+                'srp6_rt', 'unit_tls', 'x509_path_bsi', 'x509_path_rsa_pss',
+                'xmss_keygen', 'xmss_keygen_reference', 'xmss_sign', 'xmss_verify', 'xmss_verify_invalid'
+            ]
 
             disabled_tests += slow_tests
 
@@ -256,7 +273,7 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache,
     if target in ['fuzzers', 'sanitizer']:
         flags += ['--with-debug-asserts']
 
-        if target_cc in ['clang', 'gcc']:
+        if target_cc in ['clang', 'gcc', 'xcode']:
             flags += ['--enable-sanitizers=address,undefined']
         else:
             flags += ['--enable-sanitizers=address']
@@ -321,7 +338,7 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache,
                 cc_bin = 'arm-linux-gnueabihf-g++'
                 test_prefix = ['qemu-arm', '-L', '/usr/arm-linux-gnueabihf/']
                 # disable a few tests that are exceptionally slow under arm32 qemu
-                disabled_tests += ['dh_invalid', 'dlies', 'xmss_sign']
+                disabled_tests += ['dh_invalid', 'dlies', 'frodo_kat_tests', 'xmss_sign']
             elif target in ['cross-arm64', 'cross-arm64-amalgamation']:
                 flags += ['--cpu=aarch64']
                 cc_bin = 'aarch64-linux-gnu-g++'
@@ -346,7 +363,6 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache,
                 flags += ['--cpu=mips64', '--with-endian=big']
                 cc_bin = 'mips64-linux-gnuabi64-g++'
                 test_prefix = ['qemu-mips64', '-L', '/usr/mips64-linux-gnuabi64/']
-                test_cmd.remove('simd_32') # no SIMD on MIPS
             elif target in ['cross-arm32-baremetal']:
                 flags += ['--cpu=arm32', '--disable-neon', '--without-stack-protector', '--ldflags=-specs=nosys.specs']
                 cc_bin = 'arm-none-eabi-c++'
@@ -373,7 +389,7 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache,
 
         if add_boost_support(target, target_os):
             flags += ['--with-boost']
-            if target_cc == 'clang':
+            if target_cc in ['clang', 'xcode']:
                 # make sure clang ignores warnings in boost headers
                 flags += ["--extra-cxxflags=--system-header-prefix=boost/"]
 
@@ -530,7 +546,7 @@ def parse_args(args):
                       help='Set number of jobs to run in parallel (default %default)')
 
     parser.add_option('--compiler-cache', default=None, metavar='CC',
-                      help='Set a compiler cache to use (ccache, sccache)')
+                      help='Set a compiler cache to use (ccache, sccache, none)')
 
     parser.add_option('--pkcs11-lib', default=os.getenv('PKCS11_LIB'), metavar='LIB',
                       help='Set PKCS11 lib to use for testing')
@@ -602,6 +618,8 @@ def main(args=None):
             options.cc_bin = 'g++'
         elif options.cc == 'clang':
             options.cc_bin = 'clang++'
+        elif options.cc == 'xcode':
+            options.cc_bin = 'clang++'
         elif options.cc == 'msvc':
             options.cc_bin = 'cl'
         elif options.cc == "emcc":
@@ -619,6 +637,9 @@ def main(args=None):
         if options.compiler_cache:
             print("Found '%s' installed, will use it..." % (options.compiler_cache))
 
+    if options.compiler_cache == 'none':
+        options.compiler_cache = None
+
     if options.compiler_cache not in [None, 'ccache', 'sccache']:
         raise Exception("Don't know about %s as a compiler cache" % (options.compiler_cache))
 
@@ -639,6 +660,9 @@ def main(args=None):
         pylint_rc = '--rcfile=%s' % (os.path.join(root_dir, 'src/configs/pylint.rc'))
         pylint_flags = [pylint_rc, '--reports=no']
 
+        if is_running_in_github_actions():
+            pylint_flags += ["--msg-template='::warning file={path},line={line},endLine={end_line}::Pylint ({category}): {msg_id} {msg} ({symbol})'"]
+
         py_scripts = [
             'configure.py',
             'src/python/botan3.py',
@@ -663,13 +687,15 @@ def main(args=None):
             'src/editors/vscode/scripts/common.py',
             'src/editors/vscode/scripts/test.py']
 
-        full_paths = [os.path.join(root_dir, s) for s in py_scripts]
-        cmds.append([py_interp, '-m', 'pylint'] + pylint_flags + full_paths)
+        # This has to run in the repository root to generate the correct
+        # relative paths in the output. Otherwise GitHub Actions will not
+        # be able to annotate the correct files.
+        cmds.append(["indir:%s" % root_dir, py_interp, '-m', 'pylint'] + pylint_flags + py_scripts)
 
     elif target == 'format':
         cmds.append([py_interp,
                      os.path.join(root_dir, 'src/scripts/dev_tools/run_clang_format.py'),
-                     '--clang-format=clang-format-15',
+                     '--clang-format=clang-format-17',
                      '--src-dir=%s' % (os.path.join(root_dir, 'src')),
                      '--check'])
     else:
@@ -731,6 +757,10 @@ def main(args=None):
                          '-shim-path', os.path.abspath(os.path.join(build_dir, 'botan_bogo_shim')),
                          '-shim-config', os.path.abspath(os.path.join(root_dir, 'src', 'bogo_shim', 'config.json'))])
 
+        if target in ['limbo']:
+            cmds.append([py_interp, os.path.join(root_dir, 'src/scripts/run_limbo_tests.py'),
+                         os.path.join(root_dir, 'limbo.json')])
+
         if target in ['coverage', 'fuzzers']:
             cmds.append([py_interp, os.path.join(root_dir, 'src/scripts/test_fuzzers.py'),
                          os.path.join(build_dir, 'fuzzer_corpus'),
@@ -761,7 +791,7 @@ def main(args=None):
         if target in ['shared', 'coverage'] and not (options.os == 'windows' and options.cpu == 'x86'):
             cmds.append([py_interp, '-b'] + python_tests)
 
-        if target in ['shared', 'static', 'bsi', 'nist']:
+        if target in ['shared', 'static']:
             cmds.append(make_cmd + ['install'])
             build_config = os.path.join(build_dir, 'build', 'build_config.json')
             cmds.append([py_interp, os.path.join(root_dir, 'src/scripts/ci_check_install.py'), build_config])
