@@ -101,7 +101,7 @@ std::vector<Test::Result> ECC_Randomized_Tests::run() {
 
       result.start_timer();
 
-      Botan::EC_Group group(group_name);
+      auto group = Botan::EC_Group::from_name(group_name);
 
       const Botan::EC_Point pt = create_random_point(this->rng(), group);
       const Botan::BigInt& group_order = group.get_order();
@@ -241,31 +241,35 @@ class EC_Group_Tests : public Test {
          for(const std::string& group_name : Botan::EC_Group::known_named_groups()) {
             Test::Result result("EC_Group " + group_name);
 
-            const Botan::OID oid = Botan::OID::from_string(group_name);
+            const auto group = Botan::EC_Group::from_name(group_name);
 
-            const Botan::EC_Group group(oid);
-
-            result.confirm("EC_Group is known", !group.get_curve_oid().empty());
+            result.confirm("EC_Group is known", group.get_curve_oid().has_value());
             result.confirm("EC_Group is considered valid", group.verify_group(this->rng(), true));
             result.confirm("EC_Group is not considered explict encoding", !group.used_explicit_encoding());
 
             result.test_eq("EC_Group has correct bit size", group.get_p().bits(), group.get_p_bits());
             result.test_eq("EC_Group has byte size", group.get_p().bytes(), group.get_p_bytes());
 
+            result.test_eq("EC_Group has cofactor == 1", group.get_cofactor(), 1);
+
             const Botan::OID from_order = Botan::EC_Group::EC_group_identity_from_order(group.get_order());
-            result.test_eq("EC_group_identity_from_order works", from_order.to_string(), oid.to_string());
 
-            result.confirm("Same group is same", group == Botan::EC_Group(group_name));
+            result.test_eq(
+               "EC_group_identity_from_order works", from_order.to_string(), group.get_curve_oid().to_string());
 
-            const Botan::EC_Group copy(group.get_p(),
-                                       group.get_a(),
-                                       group.get_b(),
-                                       group.get_g_x(),
-                                       group.get_g_y(),
-                                       group.get_order(),
-                                       group.get_cofactor());
+            result.confirm("Same group is same", group == Botan::EC_Group::from_name(group_name));
 
-            result.confirm("Same group is same even with copy", group == copy);
+            try {
+               const Botan::EC_Group copy(group.get_curve_oid(),
+                                          group.get_p(),
+                                          group.get_a(),
+                                          group.get_b(),
+                                          group.get_g_x(),
+                                          group.get_g_y(),
+                                          group.get_order());
+
+               result.confirm("Same group is same even with copy", group == copy);
+            } catch(Botan::Invalid_Argument&) {}
 
             const auto group_der_oid = group.DER_encode(Botan::EC_Group_Encoding::NamedCurve);
             const Botan::EC_Group group_via_oid(group_der_oid);
@@ -410,11 +414,11 @@ BOTAN_REGISTER_TEST("pubkey", "ec_group", EC_Group_Tests);
 Test::Result test_decoding_with_seed() {
    Test::Result result("ECC Unit");
 
-   Botan::EC_Group secp384r1_with_seed(Test::read_data_file("x509/ecc/secp384r1_seed.pem"));
+   const auto secp384r1_with_seed = Botan::EC_Group::from_PEM(Test::read_data_file("x509/ecc/secp384r1_seed.pem"));
 
    result.confirm("decoding worked", secp384r1_with_seed.initialized());
 
-   Botan::EC_Group secp384r1("secp384r1");
+   const auto secp384r1 = Botan::EC_Group::from_name("secp384r1");
 
    result.test_eq("P-384 prime", secp384r1_with_seed.get_p(), secp384r1.get_p());
 
@@ -428,7 +432,7 @@ Test::Result test_coordinates() {
    const Botan::BigInt exp_affine_y("1373093393927139016463695321221277758035357890939");
 
    // precalculation
-   const Botan::EC_Group secp160r1("secp160r1");
+   const auto secp160r1 = Botan::EC_Group::from_name("secp160r1");
    const Botan::EC_Point& p_G = secp160r1.get_base_point();
 
    const Botan::EC_Point point_exp = secp160r1.point(exp_affine_x, exp_affine_y);
@@ -454,7 +458,7 @@ Section 2.1.2
 Test::Result test_point_mult() {
    Test::Result result("ECC Unit");
 
-   Botan::EC_Group secp160r1("secp160r1");
+   const auto secp160r1 = Botan::EC_Group::from_name("secp160r1");
    const Botan::EC_Point& p_G = secp160r1.get_base_point();
 
    Botan::BigInt d_U("0xaa374ffc3ce144e6b073307972cb6d57b2a4e982");
@@ -468,7 +472,7 @@ Test::Result test_point_mult() {
 Test::Result test_point_negative() {
    Test::Result result("ECC Unit");
 
-   Botan::EC_Group secp160r1("secp160r1");
+   const auto secp160r1 = Botan::EC_Group::from_name("secp160r1");
    const Botan::EC_Point& p_G = secp160r1.get_base_point();
 
    const Botan::EC_Point p1 = p_G * 2;
@@ -486,7 +490,7 @@ Test::Result test_point_negative() {
 Test::Result test_mult_point() {
    Test::Result result("ECC Unit");
 
-   Botan::EC_Group secp160r1("secp160r1");
+   const auto secp160r1 = Botan::EC_Group::from_name("secp160r1");
    const Botan::EC_Point& p_G = secp160r1.get_base_point();
 
    const Botan::EC_Point& p0 = p_G;
@@ -505,8 +509,8 @@ Test::Result test_mult_point() {
 Test::Result test_mixed_points() {
    Test::Result result("ECC Unit");
 
-   Botan::EC_Group secp256r1("secp256r1");
-   Botan::EC_Group secp384r1("secp384r1");
+   const auto secp256r1 = Botan::EC_Group::from_name("secp256r1");
+   const auto secp384r1 = Botan::EC_Group::from_name("secp384r1");
 
    const Botan::EC_Point& G256 = secp256r1.get_base_point();
    const Botan::EC_Point& G384 = secp384r1.get_base_point();
@@ -519,7 +523,7 @@ Test::Result test_basic_operations() {
    Test::Result result("ECC Unit");
 
    // precalculation
-   Botan::EC_Group secp160r1("secp160r1");
+   const auto secp160r1 = Botan::EC_Group::from_name("secp160r1");
    const Botan::EC_Point& p_G = secp160r1.get_base_point();
 
    const Botan::EC_Point& p0 = p_G;
@@ -554,7 +558,7 @@ Test::Result test_enc_dec_compressed_160() {
    Test::Result result("ECC Unit");
 
    // Test for compressed conversion (02/03) 160bit
-   Botan::EC_Group secp160r1("secp160r1");
+   const auto secp160r1 = Botan::EC_Group::from_name("secp160r1");
    const std::vector<uint8_t> G_comp = Botan::hex_decode("024A96B5688EF573284664698968C38BB913CBFC82");
    const Botan::EC_Point p = secp160r1.OS2ECP(G_comp);
    const std::vector<uint8_t> sv_result = p.encode(Botan::EC_Point_Format::Compressed);
@@ -566,7 +570,7 @@ Test::Result test_enc_dec_compressed_160() {
 Test::Result test_enc_dec_compressed_256() {
    Test::Result result("ECC Unit");
 
-   Botan::EC_Group group("secp256r1");
+   const auto group = Botan::EC_Group::from_name("secp256r1");
 
    const std::string G_secp_comp = "036B17D1F2E12C4247F8BCE6E563A440F277037D812DEB33A0F4A13945D898C296";
    const std::vector<uint8_t> sv_G_secp_comp = Botan::hex_decode(G_secp_comp);
@@ -595,6 +599,7 @@ Test::Result test_enc_dec_uncompressed_112() {
    const Botan::BigInt order("0x36DF0AAFD8B8D7597CA10520D04B");
    const Botan::BigInt cofactor("4");  // !
 
+   // This uses the deprecated constructor due to making use of cofactor > 1
    const Botan::EC_Group group(p, a, b, g_x, g_y, order, cofactor);
 
    const std::string G_secp_uncomp = "044BA30AB5E892B4E1649DD0928643ADCD46F5882E3747DEF36E956E97";
@@ -617,7 +622,7 @@ Test::Result test_enc_dec_uncompressed_521() {
 
    const std::vector<uint8_t> sv_G_secp_uncomp = Botan::hex_decode(G_secp_uncomp);
 
-   Botan::EC_Group group("secp521r1");
+   const auto group = Botan::EC_Group::from_name("secp521r1");
 
    Botan::EC_Point p_G = group.OS2ECP(sv_G_secp_uncomp);
 
@@ -630,21 +635,21 @@ Test::Result test_enc_dec_uncompressed_521() {
 Test::Result test_ecc_registration() {
    Test::Result result("ECC registration");
 
-   // secp112r1
-   const Botan::BigInt p("0xDB7C2ABF62E35E668076BEAD208B");
-   const Botan::BigInt a("0xDB7C2ABF62E35E668076BEAD2088");
-   const Botan::BigInt b("0x659EF8BA043916EEDE8911702B22");
+   // secp128r1
+   const Botan::BigInt p("0xfffffffdffffffffffffffffffffffff");
+   const Botan::BigInt a("0xfffffffdfffffffffffffffffffffffc");
+   const Botan::BigInt b("0xe87579c11079f43dd824993c2cee5ed3");
 
-   const Botan::BigInt g_x("0x09487239995A5EE76B55F9C2F098");
-   const Botan::BigInt g_y("0xA89CE5AF8724C0A23E0E0FF77500");
-   const Botan::BigInt order("0xDB7C2ABF62E35E7628DFAC6561C5");
+   const Botan::BigInt g_x("0x161ff7528b899b2d0c28607ca52c5b86");
+   const Botan::BigInt g_y("0xcf5ac8395bafeb13c02da292dded7a83");
+   const Botan::BigInt order("0xfffffffe0000000075a30d1b9038a115");
 
-   const Botan::OID oid("1.3.132.0.6");
+   const Botan::OID oid("1.3.132.0.28");
 
    // Creating this object implicitly registers the curve for future use ...
-   Botan::EC_Group reg_group(p, a, b, g_x, g_y, order, 1, oid);
+   Botan::EC_Group reg_group(oid, p, a, b, g_x, g_y, order);
 
-   Botan::EC_Group group(oid);
+   auto group = Botan::EC_Group::from_OID(oid);
 
    result.test_eq("Group registration worked", group.get_p(), p);
 
@@ -656,17 +661,19 @@ Test::Result test_ec_group_from_params() {
 
    Botan::EC_Group::clear_registered_curve_data();
 
-   // secp160r1 params
-   const Botan::BigInt p("0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF7FFFFFFF");
-   const Botan::BigInt a("0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF7FFFFFFC");
-   const Botan::BigInt b("0x1C97BEFC54BD7A8B65ACF89F81D4D4ADC565FA45");
+   // secp256r1
+   const Botan::BigInt p("0xFFFFFFFF00000001000000000000000000000000FFFFFFFFFFFFFFFFFFFFFFFF");
+   const Botan::BigInt a("0xFFFFFFFF00000001000000000000000000000000FFFFFFFFFFFFFFFFFFFFFFFC");
+   const Botan::BigInt b("0x5AC635D8AA3A93E7B3EBBD55769886BC651D06B0CC53B0F63BCE3C3E27D2604B");
 
-   const Botan::BigInt g_x("0x4A96B5688EF573284664698968C38BB913CBFC82");
-   const Botan::BigInt g_y("0x23A628553168947D59DCC912042351377AC5FB32");
-   const Botan::BigInt order("0x100000000000000000001F4C8F927AED3CA752257");
+   const Botan::BigInt g_x("0x6B17D1F2E12C4247F8BCE6E563A440F277037D812DEB33A0F4A13945D898C296");
+   const Botan::BigInt g_y("0x4FE342E2FE1A7F9B8EE7EB4A7C0F9E162BCE33576B315ECECBB6406837BF51F5");
+   const Botan::BigInt order("0xFFFFFFFF00000000FFFFFFFFFFFFFFFFBCE6FAADA7179E84F3B9CAC2FC632551");
 
-   const Botan::OID oid("1.3.132.0.8");
+   const Botan::OID oid("1.2.840.10045.3.1.7");
 
+   // This uses the deprecated constructor to verify we dedup even without an OID
+   // This whole test can be removed once explicit curve support is removed
    Botan::EC_Group reg_group(p, a, b, g_x, g_y, order, 1);
    result.confirm("Group has correct OID", reg_group.get_curve_oid() == oid);
 
@@ -678,19 +685,19 @@ Test::Result test_ec_group_bad_registration() {
 
    Botan::EC_Group::clear_registered_curve_data();
 
-   // secp160r1 params except with a bad B param
-   const Botan::BigInt p("0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF7FFFFFFF");
-   const Botan::BigInt a("0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF7FFFFFFC");
-   const Botan::BigInt b("0x1C97BEFC54BD7A8B65ACF89F81D4D4ADC565FA47");
+   // secp256r1 params except with a bad B param
+   const Botan::BigInt p("0xFFFFFFFF00000001000000000000000000000000FFFFFFFFFFFFFFFFFFFFFFFF");
+   const Botan::BigInt a("0xFFFFFFFF00000001000000000000000000000000FFFFFFFFFFFFFFFFFFFFFFFC");
+   const Botan::BigInt b("0x5AC635D8AA3A93E7B3EBBD55769886BC651D06B0CC53B0F63BCE3C3E27D2604C");
 
-   const Botan::BigInt g_x("0x4A96B5688EF573284664698968C38BB913CBFC82");
-   const Botan::BigInt g_y("0x23A628553168947D59DCC912042351377AC5FB32");
-   const Botan::BigInt order("0x100000000000000000001F4C8F927AED3CA752257");
+   const Botan::BigInt g_x("0x6B17D1F2E12C4247F8BCE6E563A440F277037D812DEB33A0F4A13945D898C296");
+   const Botan::BigInt g_y("0x4FE342E2FE1A7F9B8EE7EB4A7C0F9E162BCE33576B315ECECBB6406837BF51F5");
+   const Botan::BigInt order("0xFFFFFFFF00000000FFFFFFFFFFFFFFFFBCE6FAADA7179E84F3B9CAC2FC632551");
 
-   const Botan::OID oid("1.3.132.0.8");
+   const Botan::OID oid("1.2.840.10045.3.1.7");
 
    try {
-      Botan::EC_Group reg_group(p, a, b, g_x, g_y, order, 1, oid);
+      Botan::EC_Group reg_group(oid, p, a, b, g_x, g_y, order);
       result.test_failure("Should have failed");
    } catch(Botan::Invalid_Argument&) {
       result.test_success("Got expected exception");
@@ -704,28 +711,28 @@ Test::Result test_ec_group_duplicate_orders() {
 
    Botan::EC_Group::clear_registered_curve_data();
 
-   // secp160r1 params
-   const Botan::BigInt p("0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF7FFFFFFF");
-   const Botan::BigInt a("0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF7FFFFFFC");
-   const Botan::BigInt b("0x1C97BEFC54BD7A8B65ACF89F81D4D4ADC565FA45");
+   // secp256r1
+   const Botan::BigInt p("0xFFFFFFFF00000001000000000000000000000000FFFFFFFFFFFFFFFFFFFFFFFF");
+   const Botan::BigInt a("0xFFFFFFFF00000001000000000000000000000000FFFFFFFFFFFFFFFFFFFFFFFC");
+   const Botan::BigInt b("0x5AC635D8AA3A93E7B3EBBD55769886BC651D06B0CC53B0F63BCE3C3E27D2604B");
 
-   const Botan::BigInt g_x("0x4A96B5688EF573284664698968C38BB913CBFC82");
-   const Botan::BigInt g_y("0x23A628553168947D59DCC912042351377AC5FB32");
-   const Botan::BigInt order("0x100000000000000000001F4C8F927AED3CA752257");
+   const Botan::BigInt g_x("0x6B17D1F2E12C4247F8BCE6E563A440F277037D812DEB33A0F4A13945D898C296");
+   const Botan::BigInt g_y("0x4FE342E2FE1A7F9B8EE7EB4A7C0F9E162BCE33576B315ECECBB6406837BF51F5");
+   const Botan::BigInt order("0xFFFFFFFF00000000FFFFFFFFFFFFFFFFBCE6FAADA7179E84F3B9CAC2FC632551");
 
    const Botan::OID oid("1.3.6.1.4.1.25258.100.0");  // some other random OID
 
-   Botan::EC_Group reg_group(p, a, b, g_x, g_y, order, 1, oid);
+   Botan::EC_Group reg_group(oid, p, a, b, g_x, g_y, order);
    result.test_success("Registration success");
    result.confirm("Group has correct OID", reg_group.get_curve_oid() == oid);
 
    // We can now get it by OID:
-   Botan::EC_Group hc_group(oid);
+   const auto hc_group = Botan::EC_Group::from_OID(oid);
    result.confirm("Group has correct OID", hc_group.get_curve_oid() == oid);
 
-   // Existing secp160r1 unmodified:
-   const Botan::OID secp160r1("1.3.132.0.8");
-   Botan::EC_Group other_group(secp160r1);
+   // Existing secp256r1 unmodified:
+   const Botan::OID secp160r1("1.2.840.10045.3.1.7");
+   const auto other_group = Botan::EC_Group::from_OID(secp160r1);
    result.confirm("Group has correct OID", other_group.get_curve_oid() == secp160r1);
 
    return result;
