@@ -24,7 +24,7 @@ class ECDH_KAT_Tests final : public PK_Key_Agreement_Test {
       std::string default_kdf(const VarMap& /*unused*/) const override { return "Raw"; }
 
       std::unique_ptr<Botan::Private_Key> load_our_key(const std::string& group_id, const VarMap& vars) override {
-         Botan::EC_Group group(group_id);
+         const auto group = Botan::EC_Group::from_name(group_id);
          const Botan::BigInt secret = vars.get_req_bn("Secret");
          return std::make_unique<Botan::ECDH_PrivateKey>(this->rng(), group, secret);
       }
@@ -42,6 +42,14 @@ class ECDH_Keygen_Tests final : public PK_Key_Generation_Test {
       }
 
       std::string algo_name() const override { return "ECDH"; }
+
+      std::unique_ptr<Botan::Public_Key> public_key_from_raw(std::string_view keygen_params,
+                                                             std::string_view /* provider */,
+                                                             std::span<const uint8_t> raw_pk) const override {
+         const auto group = Botan::EC_Group(keygen_params);
+         const auto public_point = group.OS2ECP(raw_pk);
+         return std::make_unique<Botan::ECDH_PublicKey>(group, public_point);
+      }
 };
 
 BOTAN_REGISTER_TEST("pubkey", "ecdh_kat", ECDH_KAT_Tests);
