@@ -248,6 +248,12 @@ class Kyber_Keygen_Tests final : public PK_Key_Generation_Test {
       }
 
       std::string algo_name() const override { return "Kyber"; }
+
+      std::unique_ptr<Botan::Public_Key> public_key_from_raw(std::string_view keygen_params,
+                                                             std::string_view /* provider */,
+                                                             std::span<const uint8_t> raw_pk) const override {
+         return std::make_unique<Botan::Kyber_PublicKey>(raw_pk, Botan::KyberMode(keygen_params));
+      }
 };
 
 BOTAN_REGISTER_TEST("kyber", "kyber_keygen", Kyber_Keygen_Tests);
