@@ -18,6 +18,7 @@
    #include <botan/pubkey.h>
    #include <botan/x509_key.h>
    #include <botan/internal/fmt.h>
+   #include <botan/internal/stl_util.h>
 
    #if defined(BOTAN_HAS_HMAC_DRBG)
       #include <botan/hmac_drbg.h>
@@ -600,6 +601,23 @@ std::vector<Test::Result> PK_Key_Generation_Test::run() {
             result.confirm("Key passes self tests", key.check_key(this->rng(), true));
          } catch(Botan::Lookup_Error&) {}
 
+         const std::string name = key.algo_name();
+         result.confirm("Key has a non-empty name", !name.empty());
+
+         if(auto oid = Botan::OID::from_name(name)) {
+            result.test_success("Keys name maps to an OID");
+
+            result.test_eq("Keys name OID is the same as the object oid",
+                           oid.value().to_string(),
+                           key.object_identifier().to_string());
+         } else {
+            const bool exception = name == "Kyber" || name == "FrodoKEM" || name == "SPHINCS+";
+
+            if(!exception) {
+               result.test_failure("Keys name " + name + " does not map to an OID");
+            }
+         }
+
          result.test_gte("Key has reasonable estimated strength (lower)", key.estimated_strength(), 64);
          result.test_lt("Key has reasonable estimated strength (upper)", key.estimated_strength(), 512);
 
@@ -630,6 +648,33 @@ std::vector<Test::Result> PK_Key_Generation_Test::run() {
                            !public_key->supports_operation(Botan::PublicKeyOperation::KeyAgreement));
          }
 
+         // Test that the raw public key can be encoded. This is not supported
+         // by all algorithms; we expect Not_Implemented for these.
+         const std::vector<std::string> algos_that_dont_have_a_raw_encoding = {"RSA"};
+         try {
+            auto raw = public_key->raw_public_key_bits();
+            result.test_ne("raw_public_key_bits is not empty", raw.size(), 0);
+
+            if(public_key->supports_operation(Botan::PublicKeyOperation::KeyAgreement)) {
+               // For KEX algorithms, raw_public_key_bits must be equal to the canonical
+               // public value obtained by PK_Key_Agreement_Key::public_value().
+               const auto* ka_key = dynamic_cast<const Botan::PK_Key_Agreement_Key*>(&key);
+               result.require("is a key agreement private key", ka_key != nullptr);
+               result.test_eq("public_key_bits has same encoding", raw, ka_key->public_value());
+            }
+
+            if(auto raw_pk = public_key_from_raw(param, prov, raw)) {
+               result.test_eq("public_key has same type", raw_pk->algo_name(), public_key->algo_name());
+               result.test_eq("public_key has same encoding", raw_pk->public_key_bits(), public_key->public_key_bits());
+            }
+         } catch(const Botan::Not_Implemented&) {
+            if(!Botan::value_exists(algos_that_dont_have_a_raw_encoding, public_key->algo_name())) {
+               result.test_failure("raw_public_key_bits not implemented for " + public_key->algo_name());
+            } else {
+               result.test_note("raw_public_key_bits threw Not_Implemented as expected for " + public_key->algo_name());
+            }
+         }
+
          // Test PEM public key round trips OK
          try {
             Botan::DataSource_Memory data_src(Botan::X509::PEM_encode(key));
