@@ -41,6 +41,7 @@ std::unique_ptr<Botan::PK_Key_Agreement_Key> kex_dh() {
    auto sk = Botan::load_private_key(kex_key->algorithm_identifier(), kex_key->private_key_bits());
    auto kex_sk = dynamic_cast<Botan::PK_Key_Agreement_Key*>(sk.get());
    if(kex_sk) {
+      // NOLINTNEXTLINE(bugprone-unused-return-value)
       (void)sk.release();
       return std::unique_ptr<Botan::PK_Key_Agreement_Key>(kex_sk);
    } else {
@@ -53,6 +54,7 @@ std::unique_ptr<Botan::PK_Key_Agreement_Key> kex_ecdh() {
    auto sk = Botan::load_private_key(kex_key->algorithm_identifier(), kex_key->private_key_bits());
    auto kex_sk = dynamic_cast<Botan::PK_Key_Agreement_Key*>(sk.get());
    if(kex_sk) {
+      // NOLINTNEXTLINE(bugprone-unused-return-value)
       (void)sk.release();
       return std::unique_ptr<Botan::PK_Key_Agreement_Key>(kex_sk);
    } else {
@@ -150,8 +152,9 @@ void roundtrip_test(Test::Result& result, Ts... kex_kem_fn) {
       "expected shared secret has expected length", decryptor.shared_key_length(0), expected_shared_secret_length);
    result.test_eq("shared secret has expected length", decaps_shared_secret.size(), expected_shared_secret_length);
 
-   result.test_eq(
-      "public key bits is the sum of its parts", hybrid_public_key.public_value().size(), expected_public_key_length);
+   result.test_eq("public key bits is the sum of its parts",
+                  hybrid_public_key.raw_public_key_bits().size(),
+                  expected_public_key_length);
 }
 
 std::vector<Test::Result> hybrid_kem_keypair() {
