@@ -11,9 +11,8 @@
 
 #if defined(BOTAN_HAS_TLS) && defined(BOTAN_HAS_TLS_ASIO_STREAM) && defined(BOTAN_TARGET_OS_HAS_THREADS)
 
-   // first version to be compatible with Networking TS (N4656) and boost::beast
-   #include <boost/version.hpp>
-   #if BOOST_VERSION >= 106600
+   #include <botan/asio_compat.h>
+   #if defined(BOTAN_FOUND_COMPATIBLE_BOOST_ASIO_VERSION)
 
       #include <functional>
       #include <memory>
@@ -23,6 +22,7 @@
 
       #include <botan/asio_stream.h>
       #include <botan/auto_rng.h>
+      #include <botan/tls_messages.h>
       #include <botan/tls_session_manager_noop.h>
 
       #include <boost/asio.hpp>
@@ -44,7 +44,7 @@ using Result = Botan_Tests::Test::Result;
 const auto k_timeout = std::chrono::seconds(30);
 const auto k_endpoints = std::vector<tcp::endpoint>{tcp::endpoint{net::ip::make_address("127.0.0.1"), 8082}};
 
-enum { max_msg_length = 512 };
+constexpr size_t MAX_MSG_LENGTH = 512;
 
 std::string server_cert() {
    return Botan_Tests::Test::data_dir() + "/x509/certstor/cert1.crt";
@@ -58,24 +58,43 @@ class Timeout_Exception : public std::runtime_error {
       using std::runtime_error::runtime_error;
 };
 
-class Peer {
+class PeerCallbacks : public Botan::TLS::StreamCallbacks {
    public:
-      Peer(const std::shared_ptr<const Botan::TLS::Policy>& policy, net::io_context& ioc) :
+      void fail_on_handshake_message(const Botan::TLS::Handshake_Type msg_type, Botan::TLS::AlertType alert) {
+         m_handshake_alerts[msg_type] = alert;
+      }
+
+      void tls_inspect_handshake_msg(const Botan::TLS::Handshake_Message& msg) final {
+         if(m_handshake_alerts.contains(msg.type())) {
+            throw Botan::TLS::TLS_Exception(m_handshake_alerts[msg.type()], "Test was configured to throw");
+         }
+      }
+
+   private:
+      std::map<Botan::TLS::Handshake_Type, Botan::TLS::AlertType> m_handshake_alerts;
+};
+
+class Peer {
+   private:
+      Peer(const std::shared_ptr<const Botan::TLS::Policy>& policy,
+           net::io_context& ioc,
+           std::shared_ptr<Basic_Credentials_Manager> credentials_manager) :
             m_rng(std::make_shared<Botan::AutoSeeded_RNG>()),
-            m_credentials_manager(std::make_shared<Basic_Credentials_Manager>(true, "")),
+            m_credentials_manager(std::move(credentials_manager)),
             m_session_mgr(std::make_shared<Botan::TLS::Session_Manager_Noop>()),
+            m_callbacks(std::make_shared<PeerCallbacks>()),
             m_ctx(std::make_shared<Botan::TLS::Context>(m_credentials_manager, m_rng, m_session_mgr, policy)),
             m_timeout_timer(ioc) {}
 
+   public:
+      Peer(const std::shared_ptr<const Botan::TLS::Policy>& policy, net::io_context& ioc) :
+            Peer(policy, ioc, std::make_shared<Basic_Credentials_Manager>(true, "")) {}
+
       Peer(const std::shared_ptr<const Botan::TLS::Policy>& policy,
            net::io_context& ioc,
            const std::string& server_cert,
            const std::string& server_key) :
-            m_rng(std::make_shared<Botan::AutoSeeded_RNG>()),
-            m_credentials_manager(std::make_shared<Basic_Credentials_Manager>(server_cert, server_key)),
-            m_session_mgr(std::make_shared<Botan::TLS::Session_Manager_Noop>()),
-            m_ctx(std::make_shared<Botan::TLS::Context>(m_credentials_manager, m_rng, m_session_mgr, policy)),
-            m_timeout_timer(ioc) {}
+            Peer(policy, ioc, std::make_shared<Basic_Credentials_Manager>(server_cert, server_key)) {}
 
       Peer(const Peer& other) = delete;
       Peer(Peer&& other) = delete;
@@ -85,7 +104,7 @@ class Peer {
       // NOLINTNEXTLINE(*-exception-escape)
       virtual ~Peer() { cancel_timeout(); }
 
-      net::mutable_buffer buffer() { return net::buffer(m_data, max_msg_length); }
+      net::mutable_buffer buffer() { return net::buffer(m_data, MAX_MSG_LENGTH); }
 
       net::mutable_buffer buffer(size_t size) { return net::buffer(m_data, size); }
 
@@ -102,14 +121,14 @@ class Peer {
             return 0;
          }
 
-         return max_msg_length - bytes_transferred;
+         return MAX_MSG_LENGTH - bytes_transferred;
       }
 
       void on_timeout(std::function<void(const std::string&)> cb) { m_on_timeout = std::move(cb); }
 
       void reset_timeout(const std::string& message) {
          m_timeout_timer.expires_after(k_timeout);
-         m_timeout_timer.async_wait([=, this](const error_code& ec) {
+         m_timeout_timer.async_wait([&](const error_code& ec) {
             if(ec != net::error::operation_aborted)  // timer cancelled
             {
                if(m_on_timeout) {
@@ -139,16 +158,20 @@ class Peer {
 
       std::shared_ptr<Botan::TLS::Context> ctx() { return m_ctx; }
 
+   protected:
+      std::shared_ptr<PeerCallbacks> callbacks() { return m_callbacks; }
+
    private:
       std::shared_ptr<Botan::AutoSeeded_RNG> m_rng;
       std::shared_ptr<Basic_Credentials_Manager> m_credentials_manager;
       std::shared_ptr<Botan::TLS::Session_Manager_Noop> m_session_mgr;
+      std::shared_ptr<PeerCallbacks> m_callbacks;
       std::shared_ptr<Botan::TLS::Context> m_ctx;
       std::unique_ptr<ssl_stream> m_stream;
       net::system_timer m_timeout_timer;
       std::function<void(const std::string&)> m_on_timeout;
 
-      char m_data[max_msg_length];
+      char m_data[MAX_MSG_LENGTH];
 };
 
 class Result_Wrapper {
@@ -164,7 +187,7 @@ class Result_Wrapper {
 
       void expect_ec(const std::string& msg, const error_code& expected, const error_code& ec) {
          if(ec != expected) {
-            m_result.test_failure(msg, "Unexpected error code: " + ec.message());
+            m_result.test_failure(msg, "Unexpected error code: " + ec.message() + " expected: " + expected.message());
          } else {
             m_result.test_success(msg);
          }
@@ -205,9 +228,16 @@ class Server : public Peer,
          error_code ec;
          const auto& endpoint = k_endpoints.back();
 
+         // NOLINTNEXTLINE(bugprone-unused-return-value,cert-err33-c)
          m_acceptor.open(endpoint.protocol(), ec);
+
+         // NOLINTNEXTLINE(bugprone-unused-return-value,cert-err33-c)
          m_acceptor.set_option(net::socket_base::reuse_address(true), ec);
+
+         // NOLINTNEXTLINE(bugprone-unused-return-value,cert-err33-c)
          m_acceptor.bind(endpoint, ec);
+
+         // NOLINTNEXTLINE(bugprone-unused-return-value,cert-err33-c)
          m_acceptor.listen(net::socket_base::max_listen_connections, ec);
 
          m_result.expect_success("listen", ec);
@@ -220,6 +250,11 @@ class Server : public Peer,
 
       void move_before_accept() { m_move_before_accept = true; }
 
+      void fail_on_handshake_message(const Botan::TLS::Handshake_Type msg_type, Botan::TLS::AlertType alert) {
+         callbacks()->fail_on_handshake_message(msg_type, alert);
+         m_expected_handshake_failure = alert;
+      }
+
       Result_Wrapper result() { return m_result; }
 
    private:
@@ -232,10 +267,10 @@ class Server : public Peer,
 
          if(m_move_before_accept) {
             // regression test for #2635
-            ssl_stream s(std::move(socket), ctx());
+            ssl_stream s(std::move(socket), ctx(), callbacks());
             create_stream(std::make_unique<ssl_stream>(std::move(s)));
          } else {
-            create_stream(std::make_unique<ssl_stream>(std::move(socket), ctx()));
+            create_stream(std::make_unique<ssl_stream>(std::move(socket), ctx(), callbacks()));
          }
 
          reset_timeout("handshake");
@@ -251,8 +286,13 @@ class Server : public Peer,
       }
 
       void handle_handshake(const error_code& ec) {
-         m_result.expect_success("handshake", ec);
-         handle_write(error_code{});
+         if(!m_expected_handshake_failure) {
+            m_result.expect_success("handshake", ec);
+            handle_write(error_code{});
+         } else {
+            m_result.expect_ec("handshake", m_expected_handshake_failure.value(), ec);
+            cancel_timeout();
+         }
       }
 
       void handle_write(const error_code& ec) {
@@ -308,6 +348,7 @@ class Server : public Peer,
    private:
       tcp::acceptor m_acceptor;
       Result_Wrapper m_result;
+      std::optional<Botan::TLS::AlertType> m_expected_handshake_failure;
       bool m_short_read_expected;
 
       // regression test for #2635
@@ -325,7 +366,7 @@ class Client : public Peer {
    public:
       Client(const std::shared_ptr<const Botan::TLS::Policy>& policy, net::io_context& ioc) : Peer(policy, ioc) {
          ctx()->set_verify_callback(accept_all);
-         create_stream(std::make_unique<ssl_stream>(ioc, ctx()));
+         create_stream(std::make_unique<ssl_stream>(ioc, ctx(), callbacks()));
       }
 
       void close_socket() {
@@ -350,9 +391,9 @@ class TestBase {
             m_server(std::make_shared<Server>(server_policy, ioc, m_name)),
             m_result(m_name) {
          m_client->on_timeout(
-            [=, this](const std::string& msg) { m_result.test_failure("timeout in client during: " + msg); });
+            [&](const std::string& msg) { m_result.test_failure("timeout in client during: " + msg); });
          m_server->on_timeout(
-            [=, this](const std::string& msg) { m_result.test_failure("timeout in server during: " + msg); });
+            [&](const std::string& msg) { m_result.test_failure("timeout in server during: " + msg); });
 
          m_server->listen();
       }
@@ -752,6 +793,68 @@ class Test_Conversation_With_Move : public Test_Conversation {
       }
 };
 
+/* In this test we provoke a handshake failure early on in the server and expect
+ * the stream to handle it by providing the configured error code to the client.
+ *
+ * This is a regression test for #3778: Alerts during handshakes resulted in an
+ * immediate closure of the socket without transmitting the Alert message first.
+ */
+class Test_Handshake_Failure : public TestBase,
+                               public net::coroutine,
+                               public std::enable_shared_from_this<Test_Handshake_Failure> {
+   public:
+      Test_Handshake_Failure(net::io_context& ioc,
+                             const std::string& config_name,
+                             const std::shared_ptr<const Botan::TLS::Policy>& client_policy,
+                             const std::shared_ptr<const Botan::TLS::Policy>& server_policy) :
+            TestBase(ioc, client_policy, server_policy, "Test Handshake Failure", config_name) {
+         server()->fail_on_handshake_message(Botan::TLS::Handshake_Type::ClientHello,
+                                             Botan::TLS::Alert::HandshakeFailure);
+      }
+
+      void run(const error_code& ec) {
+         static auto test_case = &Test_Handshake_Failure::run;
+         reenter(*this) {
+            client()->reset_timeout("connect");
+            yield net::async_connect(
+               client()->stream().lowest_layer(), k_endpoints, std::bind(test_case, shared_from_this(), _1));
+            result().expect_success("connect", ec);
+
+            client()->reset_timeout("handshake");
+            yield client()->stream().async_handshake(Botan::TLS::Connection_Side::Client,
+                                                     std::bind(test_case, shared_from_this(), _1));
+            result().expect_ec("handshake", Botan::TLS::Alert::HandshakeFailure, ec);
+
+            client()->close_socket();
+            teardown();
+         }
+      }
+};
+
+class Test_Handshake_Failure_Sync : public Synchronous_Test {
+   public:
+      Test_Handshake_Failure_Sync(net::io_context& ioc,
+                                  const std::string& config_name,
+                                  const std::shared_ptr<const Botan::TLS::Policy>& client_policy,
+                                  const std::shared_ptr<const Botan::TLS::Policy>& server_policy) :
+            Synchronous_Test(ioc, client_policy, server_policy, "Test Handshake Failure Sync", config_name) {
+         server()->fail_on_handshake_message(Botan::TLS::Handshake_Type::ClientHello,
+                                             Botan::TLS::Alert::HandshakeFailure);
+      }
+
+      void run_synchronous_client() override {
+         error_code ec;
+         net::connect(client()->stream().lowest_layer(), k_endpoints, ec);
+         result().expect_success("connect", ec);
+
+         client()->stream().handshake(Botan::TLS::Connection_Side::Client, ec);
+         result().expect_ec("handshake", Botan::TLS::Alert::HandshakeFailure, ec);
+
+         client()->close_socket();
+         teardown();
+      }
+};
+
 class SystemConfiguration {
    public:
       SystemConfiguration(std::string n, const std::string& cp, const std::string& sp) :
@@ -821,10 +924,12 @@ class Tls_Stream_Integration_Tests final : public Test {
             config.run<Test_Eager_Close>(results);
             config.run<Test_Close_Without_Shutdown>(results);
             config.run<Test_No_Shutdown_Response>(results);
+            config.run<Test_Handshake_Failure>(results);
             config.run<Test_Conversation_Sync>(results);
             config.run<Test_Eager_Close_Sync>(results);
             config.run<Test_Close_Without_Shutdown_Sync>(results);
             config.run<Test_No_Shutdown_Response_Sync>(results);
+            config.run<Test_Handshake_Failure_Sync>(results);
             config.run<Test_Conversation_With_Move>(results);
          }
 
@@ -836,5 +941,5 @@ BOTAN_REGISTER_TEST("tls", "tls_stream_integration", Tls_Stream_Integration_Test
 
 }  // namespace Botan_Tests
 
-   #endif  // BOOST_VERSION
-#endif     // BOTAN_HAS_TLS && BOTAN_HAS_BOOST_ASIO
+   #endif
+#endif  // BOTAN_HAS_TLS && BOTAN_HAS_BOOST_ASIO
