@@ -1,5 +1,5 @@
 /*
-* (C) 2015,2018 Jack Lloyd
+* (C) 2015,2018,2024 Jack Lloyd
 * (C) 2016 Daniel Neus, Rohde & Schwarz Cybersecurity
 * (C) 2017 René Korthaus, Rohde & Schwarz Cybersecurity
 *
@@ -14,6 +14,7 @@
 #include <botan/internal/cpuid.h>
 #include <botan/internal/ct_utils.h>
 #include <botan/internal/fmt.h>
+#include <botan/internal/int_utils.h>
 #include <botan/internal/loadstor.h>
 #include <botan/internal/parsing.h>
 #include <botan/internal/rounding.h>
@@ -33,44 +34,134 @@ namespace Botan_Tests {
 
 namespace {
 
-class Utility_Function_Tests final : public Text_Based_Test {
+class Utility_Function_Tests final : public Test {
    public:
-      Utility_Function_Tests() : Text_Based_Test("util.vec", "In1,In2,Out") {}
+      std::vector<Test::Result> run() override {
+         std::vector<Test::Result> results;
 
-      Test::Result run_one_test(const std::string& algo, const VarMap& vars) override {
-         Test::Result result("Util " + algo);
+         results.push_back(test_checked_add());
+         results.push_back(test_checked_mul());
+         results.push_back(test_checked_cast());
+         results.push_back(test_round_up());
+         results.push_back(test_loadstore());
+         results.push_back(test_loadstore_ambiguity());
+         results.push_back(test_loadstore_fallback());
+         results.push_back(test_loadstore_constexpr());
+         return Botan::concat(results, test_copy_out_be_le());
+         return results;
+      }
 
-         if(algo == "round_up") {
-            const size_t x = vars.get_req_sz("In1");
-            const size_t to = vars.get_req_sz("In2");
+   private:
+      Test::Result test_checked_add() {
+         Test::Result result("checked_add");
 
-            result.test_eq(algo, Botan::round_up(x, to), vars.get_req_sz("Out"));
+         const size_t large = static_cast<size_t>(-5);
+         const size_t zero = 0;
 
-            try {
-               Botan::round_up(x, 0);
-               result.test_failure("round_up did not reject invalid input");
-            } catch(std::exception&) {}
-         } else if(algo == "round_down") {
-            const size_t x = vars.get_req_sz("In1");
-            const size_t to = vars.get_req_sz("In2");
+         for(int si = -15; si != 15; ++si) {
+            const size_t i = static_cast<size_t>(si);
+            auto sum1 = Botan::checked_add<size_t>(i, zero, zero, zero, large);
+            auto sum2 = Botan::checked_add<size_t>(large, zero, zero, zero, i);
 
-            result.test_eq(algo, Botan::round_down<size_t>(x, to), vars.get_req_sz("Out"));
-            result.test_eq(algo, Botan::round_down<size_t>(x, 0), x);
+            result.confirm("checked_add looks at all args", sum1 == sum2);
+
+            if(i < 5) {
+               result.test_eq("checked_add worked", sum1.value(), i + large);
+            } else {
+               result.confirm("checked_add did not return a result", !sum1.has_value());
+            }
+         }
+
+         auto& rng = Test::rng();
+
+         for(size_t i = 0; i != 100; ++i) {
+            const uint16_t x = Botan::make_uint16(rng.next_byte(), rng.next_byte());
+            const uint16_t y = Botan::make_uint16(rng.next_byte(), rng.next_byte());
+
+            const uint32_t ref = static_cast<uint32_t>(x) + y;
+
+            if(auto z = Botan::checked_add(x, y)) {
+               result.test_int_eq("checked_add adds", z.value(), ref);
+            } else {
+               result.confirm("checked_add checks", (ref >> 16) > 0);
+            }
          }
 
          return result;
       }
 
-      std::vector<Test::Result> run_final_tests() override {
-         std::vector<Test::Result> results;
+      Test::Result test_checked_mul() {
+         Test::Result result("checked_mul");
 
-         results.push_back(test_loadstore());
-         results.push_back(test_loadstore_fallback());
-         results.push_back(test_loadstore_constexpr());
-         return Botan::concat(results, test_copy_out_be_le());
+         auto& rng = Test::rng();
+
+         for(size_t i = 0; i != 100; ++i) {
+            const uint16_t x = Botan::make_uint16(rng.next_byte(), rng.next_byte());
+            const uint16_t y = Botan::make_uint16(rng.next_byte(), rng.next_byte());
+
+            const uint32_t ref = static_cast<uint32_t>(x) * y;
+
+            if(auto z = Botan::checked_mul(x, y)) {
+               result.test_int_eq("checked_mul multiplies", z.value(), ref);
+            } else {
+               result.confirm("checked_mul checks", (ref >> 16) > 0);
+            }
+         }
+
+         return result;
+      }
+
+      Test::Result test_checked_cast() {
+         Test::Result result("checked_cast");
+
+         const uint32_t large = static_cast<uint32_t>(-1);
+         const uint32_t is_16_bits = 0x8123;
+         const uint32_t is_8_bits = 0x89;
+
+         result.test_throws("checked_cast checks", [&] { Botan::checked_cast_to<uint16_t>(large); });
+         result.test_throws("checked_cast checks", [&] { Botan::checked_cast_to<uint8_t>(large); });
+
+         result.test_int_eq("checked_cast converts", Botan::checked_cast_to<uint32_t>(large), large);
+         result.test_int_eq("checked_cast converts", Botan::checked_cast_to<uint16_t>(is_16_bits), 0x8123);
+         result.test_int_eq("checked_cast converts", Botan::checked_cast_to<uint8_t>(is_8_bits), 0x89);
+
+         return result;
+      }
+
+      Test::Result test_round_up() {
+         Test::Result result("Util round_up");
+
+         // clang-format off
+         const std::vector<size_t> inputs = {
+            0, 1, 2, 3, 4, 9, 10, 32, 99, 100, 101, 255, 256, 1000, 10000,
+            65535, 65536, 65537,
+         };
+
+         const std::vector<size_t> alignments = {
+            1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 16, 32, 50, 64, 100, 512, 521,
+            1000, 1023, 1024, 1025, 10000, 65535, 65536
+         };
+         // clang-format on
+
+         for(size_t i : inputs) {
+            for(size_t m : alignments) {
+               try {
+                  const size_t z = Botan::round_up(i, m);
+
+                  result.confirm("z % m == 0", z % m == 0);
+                  result.confirm("z >= i", z >= i);
+                  result.confirm("z <= i + m", z <= i + m);
+               } catch(Botan::Exception& e) {
+                  result.test_failure(Botan::fmt("round_up({},{})", i, m), e.what());
+               }
+            }
+         }
+
+         result.test_throws("Integer overflow is detected", []() { Botan::round_up(static_cast<size_t>(-1), 1024); });
+
+         return result;
       }
 
-   private:
       using TestInt64 = Botan::Strong<uint64_t, struct TestInt64_>;
       using TestInt32 = Botan::Strong<uint32_t, struct TestInt64_>;
       using TestVectorSink = Botan::Strong<std::vector<uint8_t>, struct TestVectorSink_>;
@@ -406,6 +497,36 @@ class Utility_Function_Tests final : public Text_Based_Test {
       template <size_t N>
       using a = std::array<uint8_t, N>;
 
+      static Test::Result test_loadstore_ambiguity() {
+         // This is a regression test for a (probable) compiler bug in Xcode 15
+         // where it would fail to compile the load/store functions for size_t
+         //
+         // It seems that this platform defines uint64_t as "unsigned long long"
+         // and size_t as "unsigned long". Both are 64-bits but the compiler
+         // was unable to disambiguate the two in reverse_bytes in bswap.h
+
+         const uint32_t in32 = 0x01234567;
+         const uint64_t in64 = 0x0123456789ABCDEF;
+         const size_t inszt = 0x87654321;
+
+         Test::Result result("Util load/store ambiguity");
+         const auto out_be_32 = Botan::store_be(in32);
+         const auto out_le_32 = Botan::store_le(in32);
+         const auto out_be_64 = Botan::store_be(in64);
+         const auto out_le_64 = Botan::store_le(in64);
+         const auto out_be_szt = Botan::store_be(inszt);
+         const auto out_le_szt = Botan::store_le(inszt);
+
+         result.test_is_eq<uint32_t>("be 32", Botan::load_be<uint32_t>(out_be_32), in32);
+         result.test_is_eq<uint32_t>("le 32", Botan::load_le<uint32_t>(out_le_32), in32);
+         result.test_is_eq<uint64_t>("be 64", Botan::load_be<uint64_t>(out_be_64), in64);
+         result.test_is_eq<uint64_t>("le 64", Botan::load_le<uint64_t>(out_le_64), in64);
+         result.test_is_eq<size_t>("be szt", Botan::load_be<size_t>(out_be_szt), inszt);
+         result.test_is_eq<size_t>("le szt", Botan::load_le<size_t>(out_le_szt), inszt);
+
+         return result;
+      }
+
       static Test::Result test_loadstore_fallback() {
          // The fallback implementation is only used if we don't know the
          // endianness of the target at compile time. This makes sure that the
@@ -652,7 +773,7 @@ BOTAN_REGISTER_SMOKE_TEST("utils", "util", Utility_Function_Tests);
 class CT_Mask_Tests final : public Test {
    public:
       std::vector<Test::Result> run() override {
-         Test::Result result("CT utils");
+         Test::Result result("CT::Mask");
 
          result.test_eq_sz("CT::is_zero8", Botan::CT::Mask<uint8_t>::is_zero(0).value(), 0xFF);
          result.test_eq_sz("CT::is_zero8", Botan::CT::Mask<uint8_t>::is_zero(1).value(), 0x00);
@@ -713,7 +834,39 @@ class CT_Mask_Tests final : public Test {
       }
 };
 
-BOTAN_REGISTER_TEST("utils", "ct_utils", CT_Mask_Tests);
+BOTAN_REGISTER_TEST("utils", "ct_mask", CT_Mask_Tests);
+
+class CT_Choice_Tests final : public Test {
+   public:
+      std::vector<Test::Result> run() override {
+         Test::Result result("CT::Choice");
+
+         result.test_eq("CT::Choice::yes", Botan::CT::Choice::yes().as_bool(), true);
+         result.test_eq("CT::Choice::no", Botan::CT::Choice::no().as_bool(), false);
+
+         test_choice_from_int<uint8_t>("uint8_t", result);
+         test_choice_from_int<uint16_t>("uint16_t", result);
+         test_choice_from_int<uint32_t>("uint32_t", result);
+         test_choice_from_int<uint64_t>("uint64_t", result);
+
+         return {result};
+      }
+
+   private:
+      template <std::unsigned_integral T>
+      void test_choice_from_int(const char* type_name, Result& result) {
+         const auto tname = Botan::fmt("CT::Choice::from_int<{}>", type_name);
+         constexpr size_t tbits = sizeof(T) * 8;
+
+         result.test_eq(tname, Botan::CT::Choice::from_int<T>(0).as_bool(), false);
+         for(size_t b = 0; b != tbits; ++b) {
+            const auto choice = Botan::CT::Choice::from_int<T>(static_cast<T>(1) << b);
+            result.test_eq(tname, choice.as_bool(), true);
+         }
+      }
+};
+
+BOTAN_REGISTER_TEST("utils", "ct_choice", CT_Choice_Tests);
 
 class BitOps_Tests final : public Test {
    public:
@@ -943,8 +1096,6 @@ class Charset_Tests final : public Text_Based_Test {
          const std::vector<uint8_t> in = vars.get_req_bin("In");
          const std::vector<uint8_t> expected = vars.get_req_bin("Out");
 
-         const std::string in_str(in.begin(), in.end());
-
          std::string converted;
 
          if(type == "UCS2-UTF8") {
@@ -986,6 +1137,31 @@ class Hostname_Tests final : public Text_Based_Test {
 
 BOTAN_REGISTER_TEST("utils", "hostname", Hostname_Tests);
 
+class IPv4_Parsing_Tests final : public Text_Based_Test {
+   public:
+      IPv4_Parsing_Tests() : Text_Based_Test("utils/ipv4.vec", "IPv4") {}
+
+      Test::Result run_one_test(const std::string& status, const VarMap& vars) override {
+         Test::Result result("IPv4 parsing");
+
+         const std::string input = vars.get_req_str("IPv4");
+         const bool valid = (status == "Valid");
+
+         auto ipv4 = Botan::string_to_ipv4(input);
+
+         result.test_eq("string_to_ipv4 accepts only valid", valid, ipv4.has_value());
+
+         if(ipv4) {
+            const std::string rt = Botan::ipv4_to_string(ipv4.value());
+            result.test_eq("ipv4_to_string and string_to_ipv4 round trip", input, rt);
+         }
+
+         return result;
+      }
+};
+
+BOTAN_REGISTER_TEST("utils", "ipv4_parse", IPv4_Parsing_Tests);
+
 class ReadKV_Tests final : public Text_Based_Test {
    public:
       ReadKV_Tests() : Text_Based_Test("utils/read_kv.vec", "Input,Expected") {}
