@@ -109,6 +109,9 @@ std::unique_ptr<Botan::Private_Key> make_a_private_key(const std::string& algo,
       if(algo == "ECKCDSA" || algo == "ECGDSA") {
          return "brainpool256r1";
       }
+      if(algo == "HSS-LMS") {
+         return "SHA-256,HW(5,4),HW(5,4)";
+      }
       return "";  // default "" means choose acceptable algo-specific params
    }();
 
@@ -818,27 +821,37 @@ Test::Result test_pkcs10_ext(const Botan::Private_Key& key,
 
    Botan::X509_Cert_Options opts;
 
+   opts.dns = "main.example.org";
+   opts.more_dns.push_back("more1.example.org");
+   opts.more_dns.push_back("more2.example.org");
+
    opts.padding_scheme = sig_padding;
 
    Botan::AlternativeName alt_name;
-   alt_name.add_attribute("DNS", "example.org");
-   alt_name.add_attribute("DNS", "example.com");
-   alt_name.add_attribute("DNS", "example.net");
+   alt_name.add_attribute("DNS", "bonus.example.org");
+
+   Botan::X509_DN alt_dn;
+   alt_dn.add_attribute("X520.CommonName", "alt_cn");
+   alt_dn.add_attribute("X520.Organization", "testing");
+   alt_name.add_dn(alt_dn);
 
    opts.extensions.add(std::make_unique<Botan::Cert_Extension::Subject_Alternative_Name>(alt_name));
 
-   Botan::PKCS10_Request req = Botan::X509::create_cert_req(opts, key, hash_fn, rng);
+   const auto req = Botan::X509::create_cert_req(opts, key, hash_fn, rng);
 
-   std::vector<std::string> alt_dns_names = req.subject_alt_name().get_attribute("DNS");
+   const auto alt_dns_names = req.subject_alt_name().get_attribute("DNS");
 
-   result.test_eq("Expected number of DNS names", alt_dns_names.size(), 3);
+   result.test_eq("Expected number of DNS names", alt_dns_names.size(), 4);
 
-   // The order is not guaranteed so sort before comparing
-   std::sort(alt_dns_names.begin(), alt_dns_names.end());
+   if(alt_dns_names.size() == 4) {
+      result.test_eq("Expected DNS name 1", alt_dns_names.at(0), "bonus.example.org");
+      result.test_eq("Expected DNS name 2", alt_dns_names.at(1), "main.example.org");
+      result.test_eq("Expected DNS name 3", alt_dns_names.at(2), "more1.example.org");
+      result.test_eq("Expected DNS name 3", alt_dns_names.at(3), "more2.example.org");
+   }
 
-   result.test_eq("Expected DNS name 1", alt_dns_names.at(0), "example.com");
-   result.test_eq("Expected DNS name 2", alt_dns_names.at(1), "example.net");
-   result.test_eq("Expected DNS name 3", alt_dns_names.at(2), "example.org");
+   result.test_eq("Expected number of alt DNs", req.subject_alt_name().directory_names().size(), 1);
+   result.confirm("Alt DN is correct", *req.subject_alt_name().directory_names().begin() == alt_dn);
 
    return result;
 }
@@ -1218,7 +1231,7 @@ Test::Result test_valid_constraints(const Botan::Private_Key& key, const std::st
       result.test_eq("crl sign not permitted", crl_sign.compatible_with(key), false);
       result.test_eq("sign", sign_everything.compatible_with(key), false);
    } else if(pk_algo == "DSA" || pk_algo == "ECDSA" || pk_algo == "ECGDSA" || pk_algo == "ECKCDSA" ||
-             pk_algo == "GOST-34.10" || pk_algo == "Dilithium") {
+             pk_algo == "GOST-34.10" || pk_algo == "Dilithium" || pk_algo == "HSS-LMS") {
       // these are signature algorithms only
       result.test_eq("all constraints not permitted", all.compatible_with(key), false);
 
@@ -1357,7 +1370,8 @@ Test::Result test_x509_extensions(const Botan::Private_Key& ca_key,
    std::vector<Botan::Cert_Extension::CRL_Distribution_Points::Distribution_Point> dps;
 
    for(const auto& uri : cdp_urls) {
-      Botan::AlternativeName cdp_alt_name("", uri);
+      Botan::AlternativeName cdp_alt_name;
+      cdp_alt_name.add_uri(uri);
       Botan::Cert_Extension::CRL_Distribution_Points::Distribution_Point dp(cdp_alt_name);
 
       dps.emplace_back(dp);
@@ -1500,6 +1514,61 @@ Test::Result test_hashes(const Botan::Private_Key& key, const std::string& hash_
    return result;
 }
 
+Test::Result test_x509_tn_auth_list_extension_decode() {
+   /* cert with TNAuthList extension data was generated by asn1parse cfg:
+
+      asn1=SEQUENCE:tn_auth_list
+
+      [tn_auth_list]
+      spc=EXP:0,IA5:1001
+      range=EXP:1,SEQUENCE:TelephoneNumberRange
+      one=EXP:2,IA5:333
+
+      [TelephoneNumberRange]
+      start1=IA5:111
+      count1=INT:128
+      start2=IA5:222
+      count2=INT:256
+    */
+   const std::string filename("TNAuthList.pem");
+   Test::Result result("X509 TNAuthList decode");
+   result.start_timer();
+
+   Botan::X509_Certificate cert(Test::data_file("x509/x509test/" + filename));
+
+   using Botan::Cert_Extension::TNAuthList;
+
+   auto tn_auth_list = cert.v3_extensions().get_extension_object_as<TNAuthList>();
+
+   auto& tn_entries = tn_auth_list->entries();
+
+   result.confirm("cert has TNAuthList extension", tn_auth_list != nullptr, true);
+
+   result.test_throws("wrong telephone_number_range() accessor for spc",
+                      [&tn_entries] { tn_entries[0].telephone_number_range(); });
+   result.test_throws("wrong telephone_number() accessor for range",
+                      [&tn_entries] { tn_entries[1].telephone_number(); });
+   result.test_throws("wrong service_provider_code() accessor for one",
+                      [&tn_entries] { tn_entries[2].service_provider_code(); });
+
+   result.test_eq("spc entry type", tn_entries[0].type() == TNAuthList::Entry::ServiceProviderCode, true);
+   result.test_eq("spc entry data", tn_entries[0].service_provider_code(), "1001");
+
+   result.test_eq("range entry type", tn_entries[1].type() == TNAuthList::Entry::TelephoneNumberRange, true);
+   auto& range = tn_entries[1].telephone_number_range();
+   result.test_eq("range entries count", range.size(), 2);
+   result.test_eq("range entry 0 start data", range[0].start.value(), "111");
+   result.test_eq("range entry 0 count data", range[0].count, 128);
+   result.test_eq("range entry 1 start data", range[1].start.value(), "222");
+   result.test_eq("range entry 1 count data", range[1].count, 256);
+
+   result.test_eq("one entry type", tn_entries[2].type() == TNAuthList::Entry::TelephoneNumber, true);
+   result.test_eq("one entry data", tn_entries[2].telephone_number(), "333");
+
+   result.end_timer();
+   return result;
+}
+
 std::vector<std::string> get_sig_paddings(const std::string& sig_algo, const std::string& hash) {
    if(sig_algo == "RSA") {
       return {"EMSA3(" + hash + ")", "EMSA4(" + hash + ")"};
@@ -1510,6 +1579,8 @@ std::vector<std::string> get_sig_paddings(const std::string& sig_algo, const std
       return {"Pure"};
    } else if(sig_algo == "Dilithium") {
       return {"Randomized"};
+   } else if(sig_algo == "HSS-LMS") {
+      return {""};
    } else {
       return {};
    }
@@ -1523,7 +1594,7 @@ class X509_Cert_Unit_Tests final : public Test {
          auto& rng = this->rng();
 
          const std::string sig_algos[]{
-            "RSA", "DSA", "ECDSA", "ECGDSA", "ECKCDSA", "GOST-34.10", "Ed25519", "Ed448", "Dilithium"};
+            "RSA", "DSA", "ECDSA", "ECGDSA", "ECKCDSA", "GOST-34.10", "Ed25519", "Ed448", "Dilithium", "HSS-LMS"};
 
          for(const std::string& algo : sig_algos) {
    #if !defined(BOTAN_HAS_EMSA_PKCS1)
@@ -1639,6 +1710,7 @@ class X509_Cert_Unit_Tests final : public Test {
          results.push_back(test_x509_authority_info_access_extension());
          results.push_back(test_verify_gost2012_cert());
          results.push_back(test_parse_rsa_pss_cert());
+         results.push_back(test_x509_tn_auth_list_extension_decode());
    #endif
 
          results.push_back(test_x509_encode_authority_info_access_extension());
